# Copyright 2022 Cloudification GmbH
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


class SharesSchedulerHintsAdminTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(SharesSchedulerHintsAdminTest, cls).skip_checks()
        if not CONF.share.multi_backend:
            raise cls.skipException("Manila multi-backend tests are disabled.")
        elif len(CONF.share.backend_names) < 2:
            raise cls.skipException("For running multi-backend tests, two or "
                                    "more backend names must be configured.")
        elif any(not name for name in CONF.share.backend_names):
            raise cls.skipException("Share backend names can not be empty.")
        utils.check_skip_if_microversion_not_supported('2.67')

    @classmethod
    def resource_setup(cls):
        super(SharesSchedulerHintsAdminTest, cls).resource_setup()
        # Need for requesting pools
        cls.admin_client = cls.admin_shares_v2_client
        # create share type
        share_type = cls.create_share_type()
        cls.share_type_id = share_type['id']

    @decorators.idempotent_id('54f4dea7-890e-443b-aea5-f6108da893f0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_only_host_scheduler_hint_in_share_creation(self):
        share_a = self.create_share(share_type_id=self.share_type_id)
        share_a = self.admin_shares_v2_client.get_share(share_a['id'])['share']
        backend_a = share_a['host']
        scheduler_hint = {"only_host": "%s" % backend_a}

        # create share with hint
        share_b = self.create_share(share_type_id=self.share_type_id,
                                    scheduler_hints=scheduler_hint,
                                    cleanup_in_class=False)
        share_b = self.admin_shares_v2_client.get_share(share_b['id'])['share']
        backend_b = share_b['host']

        # verify same backends
        self.assertEqual(backend_a, backend_b)

    @decorators.idempotent_id('1dec3306-61f4-41b9-ba4a-572a9e6f5f57')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @tc.skipUnless(CONF.share.run_replication_tests,
                   'Replication tests are disabled.')
    def test_only_host_scheduler_hint_in_share_replica_creation(self):
        replication_type = CONF.share.backend_replication_type
        if replication_type not in constants.REPLICATION_TYPE_CHOICES:
            raise share_exceptions.ShareReplicationTypeException(
                replication_type=replication_type
            )
        extra_specs = self.add_extra_specs_to_dict({
            "replication_type": replication_type
        })
        replicated_share_type = self.create_share_type(
            data_utils.rand_name("replicated-shares"),
            extra_specs=extra_specs)
        share = self.create_share(
            share_type_id=replicated_share_type['id'],
            cleanup_in_class=False)
        share = self.admin_shares_v2_client.get_share(share['id'])['share']
        share_host = share['host']
        rep_domain, pools = self.get_pools_for_replication_domain(share=share)
        if len(pools) < 2:
            msg = ("Can not create valid hint due to insufficient pools.")
            raise self.skipException(msg)

        for p in pools:
            if p['name'] != share_host:
                expected_replica_host = p['name']
                scheduler_hint = {"only_host": "%s" % expected_replica_host}
                break

        # create share replica with hint
        replica = self.create_share_replica(share['id'],
                                            cleanup_in_class=False,
                                            version=LATEST_MICROVERSION,
                                            scheduler_hints=scheduler_hint)
        replica = self.admin_shares_v2_client.get_share_replica(
            replica['id'])['share_replica']
        replica_host = replica['host']

        # verify same backends
        self.assertEqual(expected_replica_host, replica_host)



# Copyright (C) 2022 China Telecom Digital Intelligence.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareTransferTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareTransferTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported(
            constants.SHARE_TRANSFER_VERSION)
        if CONF.share.multitenancy_enabled:
            raise cls.skipException(
                'Only for driver_handles_share_servers = False driver mode.')

    @classmethod
    def resource_setup(cls):
        super(ShareTransferTest, cls).resource_setup()
        # create share_type with dhss=False
        extra_specs = cls.add_extra_specs_to_dict()
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

    @decorators.idempotent_id('716e71a0-8265-4410-9170-08714095d9e8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_create_and_delete_share_transfer(self):
        # create share
        share_name = data_utils.rand_name("tempest-share-name")
        share = self.create_share(name=share_name,
                                  share_type_id=self.share_type_id,
                                  cleanup_in_class=False)

        # create share transfer
        transfer = self.shares_v2_client.create_share_transfer(
            share['id'], name='tempest_share_transfer')['transfer']
        waiters.wait_for_resource_status(
            self.shares_client, share['id'], 'awaiting_transfer')

        # check transfer exists and show transfer
        transfer_show = self.shares_v2_client.get_share_transfer(
            transfer['id'])['transfer']
        self.assertEqual(transfer_show['name'], 'tempest_share_transfer')

        # delete share transfer
        self.shares_v2_client.delete_share_transfer(transfer['id'])
        waiters.wait_for_resource_status(
            self.shares_client, share['id'], 'available')

        # check transfer not in transfer list
        transfers = self.shares_v2_client.list_share_transfers()['transfers']
        transfer_ids = [tf['id'] for tf in transfers]
        self.assertNotIn(transfer['id'], transfer_ids)

    @decorators.idempotent_id('3c2622ab-3368-4693-afb6-e60bd27e61ef')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_create_and_accept_share_transfer(self):
        # create share
        share_name = data_utils.rand_name("tempest-share-name")
        share = self.create_share(name=share_name,
                                  share_type_id=self.share_type_id)

        # create share transfer
        transfer = self.shares_v2_client.create_share_transfer(
            share['id'])['transfer']
        waiters.wait_for_resource_status(
            self.shares_client, share['id'], 'awaiting_transfer')

        # accept share transfer by alt project
        self.alt_shares_v2_client.accept_share_transfer(transfer['id'],
                                                        transfer['auth_key'])
        waiters.wait_for_resource_status(
            self.alt_shares_client, share['id'], 'available')

        # check share in alt project
        shares = self.alt_shares_v2_client.list_shares(
            detailed=True)['shares']
        share_ids = [sh['id'] for sh in shares] if shares else []
        self.assertIn(share['id'], share_ids)

        # delete the share
        self.alt_shares_v2_client.delete_share(share['id'])
        self.alt_shares_v2_client.wait_for_resource_deletion(
            share_id=share["id"])



# Copyright 2022 NetApp Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareServerMultipleSubNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareServerMultipleSubNegativeTest, cls).skip_checks()
        if not CONF.share.multitenancy_enabled:
            raise cls.skipException('Multitenancy tests are disabled.')
        if not CONF.share.run_share_server_multiple_subnet_tests:
            raise cls.skipException(
                'Share server multiple subnets and network allocation '
                'update tests are disabled.')
        utils.check_skip_if_microversion_not_supported("2.70")

    @classmethod
    def resource_setup(cls):
        super(ShareServerMultipleSubNegativeTest, cls).resource_setup()
        cls.share_network = cls.shares_v2_client.get_share_network(
            cls.shares_v2_client.share_network_id)['share_network']

    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @decorators.idempotent_id('1e2a9415-b02f-4c02-812d-bedc361f92ce')
    def test_create_share_multiple_subnets_to_unsupported_backend(self):
        extra_specs = {
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
            'share_server_multiple_subnet_support': False
        }
        share_type = self.create_share_type(extra_specs=extra_specs)
        pools = self.get_pools_matching_share_type(
            share_type, client=self.admin_shares_v2_client)
        zones = self.get_availability_zones_matching_share_type(
            share_type)
        if not pools or not zones:
            raise self.skipException("At least one backend that does not "
                                     "support adding multiple subnets into a "
                                     "share network is needed for this test.")
        extra_specs = {'pool_name': pools[0]['pool'],
                       'availability_zone': zones[0]}
        self.admin_shares_v2_client.update_share_type_extra_specs(
            share_type['id'], extra_specs)

        share_network_id = self.create_share_network(
            cleanup_in_class=True)["id"]
        default_subnet = utils.share_network_get_default_subnet(
            self.share_network)
        subnet_data = {
            'neutron_net_id': default_subnet.get('neutron_net_id'),
            'neutron_subnet_id': default_subnet.get('neutron_subnet_id'),
            'share_network_id': share_network_id,
            'availability_zone': zones[0],
            'cleanup_in_class': False
        }
        subnet1 = self.create_share_network_subnet(**subnet_data)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.shares_v2_client.delete_subnet,
                        share_network_id, subnet1['id'])
        subnet2 = self.create_share_network_subnet(**subnet_data)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.shares_v2_client.delete_subnet,
                        share_network_id, subnet2['id'])
        self.assertRaises(
            share_exceptions.ShareBuildErrorException,
            self.create_share,
            share_type_id=share_type['id'],
            share_network_id=share_network_id,
            availability_zone=zones[0],
            cleanup_in_class=False
        )



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacExportLocationsTests(rbac_base.ShareRbacBaseTests,
                                    metaclass=abc.ABCMeta):

    @classmethod
    def skip_checks(cls):
        super(ShareRbacExportLocationsTests, cls).skip_checks()
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

    @classmethod
    def setup_clients(cls):
        super(ShareRbacExportLocationsTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()
        cls.alt_project_share_v2_client = (
            cls.os_project_alt_member.share_v2.SharesV2Client())

    @abc.abstractmethod
    def test_list_export_locations(self):
        pass

    @abc.abstractmethod
    def test_show_export_location(self):
        pass


class TestProjectAdminTestsNFS(ShareRbacExportLocationsTests,
                               base.BaseSharesTest):

    credentials = ['project_admin', 'project_alt_member']
    protocol = 'nfs'

    @classmethod
    def setup_clients(cls):
        super(TestProjectAdminTestsNFS, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.persona, project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @classmethod
    def resource_setup(cls):
        super(TestProjectAdminTestsNFS, cls).resource_setup()
        share_type = cls.get_share_type()
        cls.share = cls.create_share(cls.client, share_type['id'])
        cls.alt_share = cls.create_share(
            cls.alt_project_share_v2_client, share_type['id'])

    @decorators.idempotent_id('c8d75c9f-104b-48a8-9817-17280ce516a8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_export_locations(self):
        locations = self.do_request(
            'list_share_export_locations', expected_status=200,
            share_id=self.share['id'])['export_locations']
        self.assertNotEmpty(locations)

        alt_locations = self.do_request(
            'list_share_export_locations', expected_status=200,
            share_id=self.alt_share['id'])['export_locations']
        self.assertNotEmpty(alt_locations)

    @decorators.idempotent_id('01ac6355-fcad-4af8-b0ad-748fc111051d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_show_export_location(self):
        export_location = self.share_member_client.list_share_export_locations(
            self.share['id'])['export_locations']
        self.do_request(
            'get_share_export_location', expected_status=200,
            share_id=self.share['id'],
            export_location_uuid=export_location[0]['id'])

        export_location = (
            self.alt_project_share_v2_client.list_share_export_locations(
                self.alt_share['id'])['export_locations'])
        self.do_request(
            'get_share_export_location', expected_status=200,
            share_id=self.alt_share['id'],
            export_location_uuid=export_location[0]['id'])


class TestProjectMemberTestsNFS(ShareRbacExportLocationsTests,
                                base.BaseSharesTest):

    credentials = ['project_member', 'project_alt_member']
    protocol = 'nfs'

    @classmethod
    def resource_setup(cls):
        super(TestProjectMemberTestsNFS, cls).resource_setup()
        share_type = cls.get_share_type()
        share_client = getattr(cls, 'share_member_client', cls.client)
        cls.share = cls.create_share(share_client, share_type['id'])
        cls.alt_share = cls.create_share(
            cls.alt_project_share_v2_client, share_type['id'])

    @decorators.idempotent_id('b25351bb-102f-437e-a0c1-01d926fa0a7d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_export_locations(self):
        locations = self.do_request(
            'list_share_export_locations', expected_status=200,
            share_id=self.share['id'])['export_locations']
        self.assertNotEmpty(locations)

        self.do_request(
            'list_share_export_locations', expected_status=lib_exc.Forbidden,
            share_id=self.alt_share['id'])

    @decorators.idempotent_id('95297484-1081-4225-986d-e4b3017d992f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_show_export_location(self):
        share_client = getattr(self, 'share_member_client', self.client)
        export_location = share_client.list_share_export_locations(
            self.share['id'])['export_locations']
        self.do_request(
            'get_share_export_location', expected_status=200,
            share_id=self.share['id'],
            export_location_uuid=export_location[0]['id'])

        export_location = (
            self.alt_project_share_v2_client.list_share_export_locations(
                self.alt_share['id'])['export_locations'])
        self.do_request(
            'get_share_export_location', expected_status=lib_exc.Forbidden,
            share_id=self.alt_share['id'],
            export_location_uuid=export_location[0]['id'])


class TestProjectReaderTestsNFS(TestProjectMemberTestsNFS):
    """Test suite for basic share export location operations by reader user

    In order to test certain share operations we must create a share
    resource for this. Since reader user is limited in resources creation, we
    are forced to use admin credentials, so we can test other share
    operations. In this class we use admin user to create a member user within
    reader project. That way we can perform a reader actions on this resource.
    """

    credentials = ['project_reader', 'project_admin', 'project_alt_member']

    @classmethod
    def setup_clients(cls):
        super(TestProjectReaderTestsNFS, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.os_project_admin,
            project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @classmethod
    def resource_setup(cls):
        super(TestProjectReaderTestsNFS, cls).resource_setup()
        project_member = cls.setup_user_client(
            cls.os_project_admin,
            project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('889ecfa7-89b3-4b16-a368-f55bbc6af228')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_export_locations(self):
        super(TestProjectReaderTestsNFS, self).test_list_export_locations()

    @decorators.idempotent_id('1ab5f0c2-d844-4284-9b66-1ba2aa14b835')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_show_export_location(self):
        super(TestProjectReaderTestsNFS, self).test_show_export_location()


class TestProjectAdminTestsCEPHFS(TestProjectAdminTestsNFS):
    protocol = 'cephfs'


class TestProjectMemberTestsCEPHFS(TestProjectMemberTestsNFS):
    protocol = 'cephfs'


class TestProjectReaderTestsCEPHFS(TestProjectReaderTestsNFS):
    protocol = 'cephfs'


class TestProjectAdminTestsCIFS(TestProjectAdminTestsNFS):
    protocol = 'cifs'


class TestProjectMemberTestsCIFS(TestProjectMemberTestsNFS):
    protocol = 'cifs'


class TestProjectReaderTestsCIFS(TestProjectReaderTestsNFS):
    protocol = 'cifs'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_utils import uuidutils
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF

MICROVERSION = '2.37'
QUERY_BY_TIMESTAMP_MICROVERSION = '2.52'


class UserMessageNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(UserMessageNegativeTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported(MICROVERSION)

    def setUp(self):
        super(UserMessageNegativeTest, self).setUp()
        self.message = self.create_user_message()

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API])
    @decorators.idempotent_id('cb6c0dbd-a3cb-404b-a358-7cec3596aff4')
    def test_show_message_of_other_tenants(self):
        self.assertRaises(lib_exc.NotFound,
                          self.alt_shares_v2_client.get_message,
                          self.message['id'])

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API])
    @decorators.idempotent_id('f493a1f9-43e1-4a85-a673-8520d5a81f68')
    def test_show_nonexistent_message(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.get_message,
                          str(uuidutils.generate_uuid()))

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API])
    @decorators.idempotent_id('2f5be1aa-974b-4f6a-ae3a-084578e64f82')
    def test_delete_message_of_other_tenants(self):
        self.assertRaises(lib_exc.NotFound,
                          self.alt_shares_v2_client.delete_message,
                          self.message['id'])

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API])
    @decorators.idempotent_id('9029cc9f-b904-4e58-b268-adf7a93cc1f1')
    def test_delete_nonexistent_message(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.delete_message,
                          str(uuidutils.generate_uuid()))

    @decorators.attr(type=[base.TAG_NEGATIVE, base.TAG_API])
    @utils.skip_if_microversion_not_supported(QUERY_BY_TIMESTAMP_MICROVERSION)
    @decorators.idempotent_id('03e80563-1a36-408e-baa8-0e3ed46f7a0a')
    def test_list_messages_with_invalid_time_format(self):
        params_key = ['created_since', 'created_before']
        for key in params_key:
            params = {key: 'invalid_time'}
            self.assertRaises(lib_exc.BadRequest,
                              self.shares_v2_client.list_messages,
                              params=params)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


class ShareNetworkSubnetMetadataNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareNetworkSubnetMetadataNegativeTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported("2.78")

    @classmethod
    def resource_setup(cls):
        super(ShareNetworkSubnetMetadataNegativeTest, cls).resource_setup()
        # create share_network and subnet
        cls.share_network = cls.create_share_network()
        az = cls.shares_v2_client.list_availability_zones(
            )['availability_zones'][0]
        cls.az_name = az['name']

        data = utils.generate_subnet_data()
        data['share_network_id'] = cls.share_network['id']
        data['availability_zone'] = cls.az_name

        cls.subnet = cls.create_share_network_subnet(**data)

    @decorators.idempotent_id('852d080f-16f3-48c4-af26-d8440be9801b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_metadata_to_unexisting_subnet(self):
        share_network = self.create_share_network()
        md = {"key1": "value1", "key2": "value2", }
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.set_metadata,
                          "wrong_subnet_id", md, resource="subnet",
                          parent_resource="share-networks",
                          parent_id=share_network['id'])

    @decorators.idempotent_id('dfe93a02-43cc-458c-92e5-ede6c2c2e597')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_all_metadata_to_unexisting_subnet(self):
        share_network = self.create_share_network()
        md = {"key1": "value1", "key2": "value2", }
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.update_all_metadata,
                          "wrong_subnet_id", md, resource="subnet",
                          parent_resource="share-networks",
                          parent_id=share_network['id'])

    @decorators.idempotent_id('fc561d8e-a2df-468f-a760-7b5d340b4b29')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_metadata_with_empty_key(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.set_metadata,
                          self.subnet['id'], {"": "value"}, resource="subnet",
                          parent_resource="share-networks",
                          parent_id=self.share_network['id'])

    @decorators.idempotent_id('b6066659-d635-4f24-9a65-fa3a132fd5ad')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_upd_metadata_with_empty_key(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.update_all_metadata,
                          self.subnet['id'], {"": "value"}, resource="subnet",
                          parent_resource="share-networks",
                          parent_id=self.share_network['id'])

    @decorators.idempotent_id('a1ea51a6-01a4-4b12-8508-04e45fdabb13')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_metadata_with_too_big_key(self):
        too_big_key = "x" * 256
        md = {too_big_key: "value"}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.set_metadata,
                          self.subnet['id'], md, resource="subnet",
                          parent_resource="share-networks",
                          parent_id=self.share_network['id'])

    @decorators.idempotent_id('a02e3970-a3d3-4c0f-a7b8-2ea3f4459214')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_upd_metadata_with_too_big_key(self):
        too_big_key = "x" * 256
        md = {too_big_key: "value"}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.update_all_metadata,
                          self.subnet['id'], md, resource="subnet",
                          parent_resource="share-networks",
                          parent_id=self.share_network['id'])

    @decorators.idempotent_id('f086589e-81a2-4d73-9fb0-9dd235cd35f1')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_metadata_with_too_big_value(self):
        too_big_value = "x" * 1024
        md = {"key": too_big_value}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.set_metadata,
                          self.subnet['id'], md, resource="subnet",
                          parent_resource="share-networks",
                          parent_id=self.share_network['id'])

    @decorators.idempotent_id('4693e944-4b85-4655-8245-f1da2b5f9ff3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_upd_metadata_with_too_big_value(self):
        too_big_value = "x" * 1024
        md = {"key": too_big_value}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.update_all_metadata,
                          self.subnet['id'], md, resource="subnet",
                          parent_resource="share-networks",
                          parent_id=self.share_network['id'])

    @decorators.idempotent_id('30e82c73-edd3-4f23-8b8f-c38e67668381')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_unexisting_metadata(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.delete_metadata,
                          self.subnet['id'], "wrong_key", resource="subnet",
                          parent_resource="share-networks",
                          parent_id=self.share_network['id'])



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base


class ShareRbacAvailabilityZonesTests(rbac_base.ShareRbacBaseTests,
                                      metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(ShareRbacAvailabilityZonesTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()

    @abc.abstractmethod
    def test_list_availability_zones(self):
        pass


class TestProjectAdminTestsNFS(ShareRbacAvailabilityZonesTests,
                               base.BaseSharesTest):

    credentials = ['project_admin']

    @decorators.idempotent_id('87d9bb1c-f4de-40e5-8f25-05a6e1055c0b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_availability_zones(self):
        self.do_request('list_availability_zones', expected_status=200)


class TestProjectMemberTestsNFS(ShareRbacAvailabilityZonesTests,
                                base.BaseSharesTest):

    credentials = ['project_member']

    @decorators.idempotent_id('ee2db349-176a-47bc-a20d-5ba9b5f8a813')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_availability_zones(self):
        self.do_request('list_availability_zones', expected_status=200)


class TestProjectReaderTestsNFS(ShareRbacAvailabilityZonesTests,
                                base.BaseSharesTest):

    credentials = ['project_reader']

    @decorators.idempotent_id('a095fac8-ae62-4be7-8a3e-b0fc1bc71348')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_availability_zones(self):
        self.do_request('list_availability_zones', expected_status=200)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base


@ddt.ddt
class SharesMetadataAPIOnlyNegativeTest(base.BaseSharesTest):

    @decorators.idempotent_id('22aecf50-0d98-4b97-82b8-599559f7692f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.data(True, False)
    def test_try_set_metadata_to_unexisting_share(self, is_v2_client):
        md = {"key1": "value1", "key2": "value2", }
        client = self.shares_v2_client if is_v2_client else self.shares_client
        self.assertRaises(lib_exc.NotFound,
                          client.set_metadata,
                          "wrong_share_id", md)

    @decorators.idempotent_id('7df0acd7-03f8-45c4-8c72-eb6932af70b1')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.data(True, False)
    def test_try_update_all_metadata_for_unexisting_share(self, is_v2_client):
        md = {"key1": "value1", "key2": "value2", }
        client = self.shares_v2_client if is_v2_client else self.shares_client
        self.assertRaises(lib_exc.NotFound,
                          client.update_all_metadata,
                          "wrong_share_id", md)


@ddt.ddt
class SharesMetadataNegativeTest(base.BaseSharesMixedTest):
    @classmethod
    def resource_setup(cls):
        super(SharesMetadataNegativeTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id)

    @decorators.idempotent_id('0cb7f160-4fa4-4d30-8a46-373ddae5844d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_set_metadata_with_empty_key(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.set_metadata,
                          self.share["id"], {"": "value"})

    @decorators.idempotent_id('759ca34d-1c87-43f3-8da2-8e1d373049ac')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_update_metadata_with_empty_key(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.update_all_metadata,
                          self.share["id"], {"": "value"})

    @decorators.idempotent_id('94c7ebb3-14c3-4ff1-9839-ae3acb318cd0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_set_metadata_with_too_big_key(self):
        too_big_key = "x" * 256
        md = {too_big_key: "value"}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.set_metadata,
                          self.share["id"], md)

    @decorators.idempotent_id('33ef3047-6ca3-4547-a681-b52314382dcb')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_update_metadata_with_too_big_key(self):
        too_big_key = "x" * 256
        md = {too_big_key: "value"}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.update_all_metadata,
                          self.share["id"], md)

    @decorators.idempotent_id('1114970a-1b45-4c56-b20a-e13e1764e3c4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_set_metadata_with_too_big_value(self):
        too_big_value = "x" * 1024
        md = {"key": too_big_value}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.set_metadata,
                          self.share["id"], md)

    @decorators.idempotent_id('c2eddcf0-cf81-4f9f-b06d-c9165ab8553e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_update_metadata_with_too_big_value(self):
        too_big_value = "x" * 1024
        md = {"key": too_big_value}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.update_all_metadata,
                          self.share["id"], md)

    @decorators.idempotent_id('14df3262-5a2b-4de4-b335-422329b22b07')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_delete_unexisting_metadata(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.delete_metadata,
                          self.share["id"], "wrong_key")



# Copyright 2014 mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class SharesNFSTest(base.BaseSharesMixedTest):
    """Covers share functionality, that is related to NFS share type."""
    protocol = "nfs"

    @classmethod
    def skip_checks(cls):
        super(SharesNFSTest, cls).skip_checks()
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

    @classmethod
    def resource_setup(cls):
        super(SharesNFSTest, cls).resource_setup()
        # create share_type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

    @decorators.idempotent_id('21ad41fb-04cf-493c-bc2f-66c80220898b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_create_get_delete_share(self):

        share = self.create_share(self.protocol,
                                  share_type_id=self.share_type_id)
        detailed_elements = {'name', 'id', 'availability_zone',
                             'description', 'project_id',
                             'created_at', 'share_proto', 'metadata',
                             'size', 'snapshot_id', 'share_network_id',
                             'status', 'share_type', 'volume_type', 'links',
                             'is_public'}
        msg = (
            "At least one expected element missing from share "
            "response. Expected %(expected)s, got %(actual)s." % {
                "expected": detailed_elements,
                "actual": share.keys(),
            }
        )
        self.assertTrue(detailed_elements.issubset(share.keys()), msg)
        self.assertFalse(share['is_public'])

        # The 'status' of the share returned by the create API must be
        # set and have value either 'creating' or
        # 'available' (if share creation is really fast as in
        # case of Dummy driver).
        self.assertIn(share['status'], ('creating', 'available'))

        # Get share using v 2.1 - we expect key 'snapshot_support' to be absent
        share_get = self.shares_v2_client.get_share(
            share['id'], version='2.1')['share']
        detailed_elements.add('export_location')
        self.assertTrue(detailed_elements.issubset(share_get.keys()), msg)

        # Get share using v 2.2 - we expect key 'snapshot_support' to exist
        share_get = self.shares_v2_client.get_share(
            share['id'], version='2.2')['share']
        detailed_elements.add('snapshot_support')
        self.assertTrue(detailed_elements.issubset(share_get.keys()), msg)

        if utils.is_microversion_supported('2.9'):
            # Get share using v 2.9 - key 'export_location' is expected
            # to be absent
            share_get = self.shares_v2_client.get_share(
                share['id'], version='2.9')['share']
            detailed_elements.remove('export_location')
            self.assertTrue(detailed_elements.issubset(share_get.keys()), msg)

        # In v 2.11 and beyond, we expect key 'replication_type' in the
        # share data returned by the share create API.
        if utils.is_microversion_supported('2.11'):
            detailed_elements.add('replication_type')
            self.assertTrue(detailed_elements.issubset(share.keys()), msg)

        # In v 2.16 and beyond, we add user_id in show/create/manage
        # share echo.
        if utils.is_microversion_supported('2.16'):
            detailed_elements.add('user_id')
            self.assertTrue(detailed_elements.issubset(share.keys()), msg)

        # In v 2.24 and beyond, we add create_share_from_snapshot_support in
        # show/create/manage share echo.
        if utils.is_microversion_supported('2.24'):
            detailed_elements.add('create_share_from_snapshot_support')
            self.assertTrue(detailed_elements.issubset(share.keys()), msg)

        # In v 2.54 and beyond, we expect key 'progress' in the share data
        # returned by the share create API.
        if utils.is_microversion_supported('2.54'):
            detailed_elements.add('progress')
            self.assertTrue(detailed_elements.issubset(share.keys()), msg)

        # Delete share
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.get_share,
                          share['id'])

    @decorators.idempotent_id('775f8f87-5727-4bb7-b69f-9ce6b9bdb140')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_create_delete_snapshot(self):
        extra_specs = {'snapshot_support': True}
        share_type = self.create_share_type(extra_specs=extra_specs,
                                            cleanup_in_class=False)
        share = self.create_share(self.protocol,
                                  share_type_id=share_type['id'],
                                  cleanup_in_class=False)

        # create snapshot
        snap = self.create_snapshot_wait_for_active(share["id"])

        detailed_elements = {'name', 'id', 'description',
                             'created_at', 'share_proto', 'size', 'share_size',
                             'share_id', 'status', 'links'}
        msg = (
            "At least one expected element missing from share "
            "response. Expected %(expected)s, got %(actual)s." % {
                "expected": detailed_elements,
                "actual": snap.keys(),
            }
        )
        self.assertTrue(detailed_elements.issubset(snap.keys()), msg)

        # In v2.17 and beyond, we expect user_id and project_id keys
        if utils.is_microversion_supported('2.17'):
            detailed_elements.update({'user_id', 'project_id'})
            self.assertTrue(detailed_elements.issubset(snap.keys()), msg)
        else:
            self.assertNotIn('user_id', detailed_elements)
            self.assertNotIn('project_id', detailed_elements)

        # In v2.73 and beyond, we expect metadata key
        if utils.is_microversion_supported('2.73'):
            detailed_elements.update({'metadata'})
            self.assertTrue(detailed_elements.issubset(snap.keys()), msg)
        else:
            self.assertNotIn('metadata', detailed_elements)

        # delete snapshot
        self.shares_client.delete_snapshot(snap["id"])
        self.shares_client.wait_for_resource_deletion(snapshot_id=snap["id"])
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.get_snapshot, snap['id'])

    @decorators.idempotent_id('8a14831d-ad1f-447f-b5de-2b8a233b24c0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Create share from snapshot tests are disabled.")
    def test_create_share_from_snapshot(self):
        # If multitenant driver used, share_network will be provided by default
        extra_specs = {
            'snapshot_support': True,
            'create_share_from_snapshot_support': True,
        }
        share_type = self.create_share_type(extra_specs=extra_specs,
                                            cleanup_in_class=False)
        share = self.create_share(self.protocol,
                                  share_type_id=share_type['id'],
                                  cleanup_in_class=False)

        # create snapshot
        snap = self.create_snapshot_wait_for_active(share["id"],
                                                    cleanup_in_class=False)

        # create share from snapshot
        s2 = self.create_share(self.protocol,
                               share_type_id=share_type['id'],
                               snapshot_id=snap["id"],
                               cleanup_in_class=False)

        # The 'status' of the share returned by the create API must be
        # set and have value either 'creating' or
        # 'available' (if share creation is really fast as in
        # case of Dummy driver).
        self.assertIn(s2['status'], ('creating', 'available'))

        # verify share, created from snapshot
        get = self.shares_client.get_share(s2["id"])['share']
        msg = ("Expected snapshot_id %s as "
               "source of share %s" % (snap["id"], get["snapshot_id"]))
        self.assertEqual(get["snapshot_id"], snap["id"], msg)

    @decorators.idempotent_id('c609c0b2-d649-4ca3-8334-629b213f5c72')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(not CONF.share.multitenancy_enabled,
                      "Only for multitenancy.")
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Create share from snapshot tests are disabled.")
    def test_create_share_from_snapshot_share_network_not_provided(self):
        # We expect usage of share network from parent's share
        # when creating share from snapshot using a driver that supports
        # multi-tenancy.

        extra_specs = {
            'snapshot_support': True,
            'create_share_from_snapshot_support': True,
        }
        share_type = self.create_share_type(extra_specs=extra_specs,
                                            cleanup_in_class=False)
        share = self.create_share(self.protocol,
                                  share_type_id=share_type['id'],
                                  cleanup_in_class=False)

        # get parent share
        parent = self.shares_client.get_share(share["id"])['share']

        # create snapshot
        snap = self.create_snapshot_wait_for_active(share["id"],
                                                    cleanup_in_class=False)

        # create share from snapshot
        child = self.create_share(self.protocol,
                                  snapshot_id=snap["id"],
                                  cleanup_in_class=False)

        # The 'status' of the share returned by the create API must be
        # set and have value either 'creating' or
        # 'available' (if share creation is really fast as in
        # case of Dummy driver).
        self.assertIn(child['status'], ('creating', 'available'))

        # verify share, created from snapshot
        get = self.shares_client.get_share(child["id"])['share']
        keys = {
            "share": share["id"],
            "actual_sn": get["share_network_id"],
            "expected_sn": parent["share_network_id"],
        }
        msg = ("Expected share_network_id %(expected_sn)s for "
               "share %(share)s, but %(actual_sn)s found." % keys)
        self.assertEqual(
            get["share_network_id"], parent["share_network_id"], msg)


class SharesCIFSTest(SharesNFSTest):
    """Covers share functionality, that is related to CIFS share type."""
    protocol = "cifs"


class SharesGLUSTERFSTest(SharesNFSTest):
    """Covers share functionality that is related to GLUSTERFS share type."""
    protocol = "glusterfs"


class SharesHDFSTest(SharesNFSTest):
    """Covers share functionality that is related to HDFS share type."""
    protocol = "hdfs"


class SharesCephFSTest(SharesNFSTest):
    """Covers share functionality that is related to CEPHFS share type."""
    protocol = "cephfs"


class SharesMapRFSTest(SharesNFSTest):
    """Covers share functionality that is related to MapRFS share type."""
    protocol = "maprfs"



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


class ShareSnapshotMetadataTest(base.BaseSharesMixedTest):
    @classmethod
    def skip_checks(cls):
        super(ShareSnapshotMetadataTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported("2.73")
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')

    @classmethod
    def resource_setup(cls):
        super(ShareSnapshotMetadataTest, cls).resource_setup()
        # create share_type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        if CONF.share.capability_create_share_from_snapshot_support:
            extra_specs.update({'create_share_from_snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share_name = data_utils.rand_name("tempest-share-name")
        cls.share_desc = data_utils.rand_name("tempest-share-description")
        cls.share = cls.create_share(
            name=cls.share_name,
            description=cls.share_desc,
            share_type_id=cls.share_type_id,
        )
        cls.share_id = cls.share["id"]

        # create snapshot
        cls.snap_name = data_utils.rand_name("tempest-snapshot-name")
        cls.snap_desc = data_utils.rand_name(
            "tempest-snapshot-description")
        cls.snap = cls.create_snapshot_wait_for_active(
            cls.share_id, cls.snap_name, cls.snap_desc)
        cls.snap_id = cls.snap['id']

    def _verify_snapshot_metadata(self, snapshot, md):

        # get metadata of snapshot
        metadata = self.shares_v2_client.get_metadata(
            snapshot['id'], resource="snapshot")['metadata']

        # verify metadata
        self.assertEqual(md, metadata)

        # verify metadata items
        for key in md:
            get_value = self.shares_v2_client.get_metadata_item(
                snapshot['id'], key, resource="snapshot")['meta']
            self.assertEqual(md[key], get_value[key])

    @decorators.idempotent_id('5d537913-ce6f-4771-beb2-84e2390b06d3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_in_snapshot_creation(self):

        md = {"key1": "value1", "key2": "value2", }

        # create snapshot with metadata
        snapshot = self.create_snapshot_wait_for_active(
            share_id=self.share_id, metadata=md,
            cleanup_in_class=False)

        # verify metadata
        self._verify_snapshot_metadata(snapshot, md)

    @decorators.idempotent_id('7cbdf3c5-fb72-4ea5-9e60-ba50bad68ee9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_get_delete_metadata(self):

        md = {"key3": "value3", "key4": "value4", "key.5.1": "value.5"}

        # create snapshot
        snapshot = self.create_snapshot_wait_for_active(
            share_id=self.share_id,
            cleanup_in_class=False)

        # set metadata
        self.shares_v2_client.set_metadata(
            snapshot['id'], md, resource="snapshot")

        # verify metadata
        self._verify_snapshot_metadata(snapshot, md)

        # delete metadata
        for key in md.keys():
            self.shares_v2_client.delete_metadata(
                snapshot['id'], key, resource="snapshot")

        # verify deletion of metadata
        get_metadata = self.shares_v2_client.get_metadata(
            snapshot['id'], resource="snapshot")['metadata']
        self.assertEmpty(get_metadata)

    @decorators.idempotent_id('23ec837d-1b50-499c-bbb9-a7bde843c9e8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_not_delete_pre_metadata(self):
        md1 = {"key9": "value9", "key10": "value10", }
        md2 = {"key11": "value11", "key12": "value12", }

        # create snapshot
        snapshot = self.create_snapshot_wait_for_active(
            share_id=self.share_id,
            cleanup_in_class=False)

        # set metadata
        self.shares_v2_client.set_metadata(
            snapshot['id'], md1, resource="snapshot")

        # verify metadata
        self._verify_snapshot_metadata(snapshot, md1)

        # set metadata again
        self.shares_v2_client.set_metadata(
            snapshot['id'], md2, resource="snapshot")

        # verify metadata
        md1.update(md2)
        md = md1

        # verify metadata
        self._verify_snapshot_metadata(snapshot, md)

        # delete metadata
        for key in md.keys():
            self.shares_v2_client.delete_metadata(
                snapshot['id'], key, resource="snapshot")

        # verify deletion of metadata
        get_metadata = self.shares_v2_client.get_metadata(
            snapshot['id'], resource="snapshot")['metadata']
        self.assertEmpty(get_metadata)

    @decorators.idempotent_id('b7a00be5-3dd1-4d25-8723-c662581c923f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_key_already_exist(self):
        md1 = {"key9": "value9", "key10": "value10", }
        md2 = {"key9": "value13", "key11": "value11", }

        # create snapshot
        snapshot = self.create_snapshot_wait_for_active(
            share_id=self.share_id,
            cleanup_in_class=False)

        # set metadata
        self.shares_v2_client.set_metadata(
            snapshot['id'], md1, resource="snapshot")

        # verify metadata
        self._verify_snapshot_metadata(snapshot, md1)

        # set metadata again
        self.shares_v2_client.set_metadata(
            snapshot['id'], md2, resource="snapshot")

        # verify metadata
        md1.update(md2)
        self._verify_snapshot_metadata(snapshot, md1)

        # delete metadata
        for key in md1.keys():
            self.shares_v2_client.delete_metadata(
                snapshot['id'], key, resource="snapshot")

        # verify deletion of metadata
        get_metadata = self.shares_v2_client.get_metadata(
            snapshot['id'], resource="snapshot")['metadata']
        self.assertEmpty(get_metadata)

    @decorators.idempotent_id('90120310-07a9-43f4-9d5e-38d0a3f2f5bb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_and_update_metadata_by_key(self):

        md1 = {"key5": "value5", "key6": "value6", }
        md2 = {"key7": "value7", "key8": "value8", }

        # create snapshot
        snapshot = self.create_snapshot_wait_for_active(
            share_id=self.share_id,
            cleanup_in_class=False)

        # set metadata
        self.shares_v2_client.set_metadata(
            snapshot['id'], md1, resource="snapshot")

        # update metadata
        self.shares_v2_client.update_all_metadata(
            snapshot['id'], md2, resource="snapshot")

        # verify metadata
        self._verify_snapshot_metadata(snapshot, md2)

    @decorators.idempotent_id('8963b7ae-db3a-476e-b0c7-29023e7aa321')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_min_size_key(self):
        data = {"k": "value"}

        self.shares_v2_client.set_metadata(self.snap_id,
                                           data, resource="snapshot")

        body_get = self.shares_v2_client.get_metadata(
            self.snap_id, resource="snapshot")['metadata']
        self.assertEqual(data['k'], body_get.get('k'))

    @decorators.idempotent_id('dc226070-5820-4df2-a30a-9dfb2f037a4b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_max_size_key(self):
        max_key = "k" * 255
        data = {max_key: "value"}

        self.shares_v2_client.set_metadata(self.snap_id,
                                           data, resource="snapshot")

        body_get = self.shares_v2_client.get_metadata(
            self.snap_id, resource="snapshot")['metadata']
        self.assertIn(max_key, body_get)
        self.assertEqual(data[max_key], body_get.get(max_key))

    @decorators.idempotent_id('940c283f-4f43-4122-86e8-32230da81886')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_min_size_value(self):
        data = {"key": "v"}

        self.shares_v2_client.set_metadata(self.snap_id,
                                           data, resource="snapshot")

        body_get = self.shares_v2_client.get_metadata(
            self.snap_id, resource="snapshot")['metadata']
        self.assertEqual(data['key'], body_get['key'])

    @decorators.idempotent_id('85c480bc-0ffa-43e1-bc0a-284c5641996d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_max_size_value(self):
        max_value = "v" * 1023
        data = {"key": max_value}

        self.shares_v2_client.set_metadata(self.snap_id,
                                           data, resource="snapshot")

        body_get = self.shares_v2_client.get_metadata(
            self.snap_id, resource="snapshot")['metadata']
        self.assertEqual(data['key'], body_get['key'])

    @decorators.idempotent_id('c42335ae-ee90-4b73-b022-51c0a9bc301d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_upd_metadata_min_size_key(self):
        data = {"k": "value"}

        self.shares_v2_client.update_all_metadata(self.snap_id,
                                                  data, resource="snapshot")

        body_get = self.shares_v2_client.get_metadata(
            self.snap_id, resource="snapshot")['metadata']
        self.assertEqual(data, body_get)

    @decorators.idempotent_id('1b5f06b0-bbff-49d1-8a4b-6e912039e2ba')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_upd_metadata_max_size_key(self):
        max_key = "k" * 255
        data = {max_key: "value"}

        self.shares_v2_client.update_all_metadata(self.snap_id,
                                                  data, resource="snapshot")

        body_get = self.shares_v2_client.get_metadata(
            self.snap_id, resource="snapshot")['metadata']
        self.assertEqual(data, body_get)

    @decorators.idempotent_id('849fdcd4-9b4c-4aea-833a-240d7d06966b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_upd_metadata_min_size_value(self):
        data = {"key": "v"}

        self.shares_v2_client.update_all_metadata(self.snap_id,
                                                  data, resource="snapshot")

        body_get = self.shares_v2_client.get_metadata(
            self.snap_id, resource="snapshot")['metadata']
        self.assertEqual(data, body_get)

    @decorators.idempotent_id('fdfbe469-6403-41de-b909-c4c13fc57407')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_upd_metadata_max_size_value(self):
        max_value = "v" * 1023
        data = {"key": max_value}

        self.shares_v2_client.update_all_metadata(self.snap_id,
                                                  data, resource="snapshot")

        body_get = self.shares_v2_client.get_metadata(
            self.snap_id, resource="snapshot")['metadata']
        self.assertEqual(data, body_get)



# Copyright 2015 Yogesh Kshirsagar
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion
_MIN_SUPPORTED_MICROVERSION = '2.11'
SUMMARY_KEYS = ['share_id', 'id', 'replica_state', 'status']
DETAIL_KEYS = SUMMARY_KEYS + ['availability_zone', 'updated_at',
                              'share_network_id', 'created_at']


@ddt.ddt
class ReplicationTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ReplicationTest, cls).skip_checks()
        if not CONF.share.run_replication_tests:
            raise cls.skipException('Replication tests are disabled.')

        utils.check_skip_if_microversion_not_supported(
            _MIN_SUPPORTED_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ReplicationTest, cls).resource_setup()
        # Create share_type
        name = data_utils.rand_name(constants.TEMPEST_MANILA_PREFIX)
        cls.admin_client = cls.admin_shares_v2_client
        cls.replication_type = CONF.share.backend_replication_type
        cls.multitenancy_enabled = (
            utils.replication_with_multitenancy_support())

        if cls.replication_type not in constants.REPLICATION_TYPE_CHOICES:
            raise share_exceptions.ShareReplicationTypeException(
                replication_type=cls.replication_type
            )
        cls.extra_specs = cls.add_extra_specs_to_dict(
            {"replication_type": cls.replication_type})
        cls.share_type = cls.create_share_type(
            name,
            extra_specs=cls.extra_specs,
            client=cls.admin_client)

        cls.zones = cls.get_availability_zones_matching_share_type(
            cls.share_type, client=cls.admin_client)
        cls.share_zone = cls.zones[0]
        cls.replica_zone = cls.zones[-1]

        # Create share with above share_type
        cls.creation_data = {'kwargs': {
            'share_type_id': cls.share_type['id'],
            'availability_zone': cls.share_zone,
        }}
        cls.sn_id = None
        if cls.multitenancy_enabled:
            cls.share_network = cls.shares_v2_client.get_share_network(
                cls.shares_v2_client.share_network_id)['share_network']
            cls.creation_data['kwargs'].update({
                'share_network_id': cls.share_network['id']})
            cls.sn_id = cls.share_network['id']

        # Data for creating shares in parallel
        data = [cls.creation_data, cls.creation_data]
        cls.shares = cls.create_shares(data)
        cls.shares = [cls.shares_v2_client.get_share(s['id'])['share'] for s in
                      cls.shares]
        cls.instance_id1 = cls._get_instance(cls.shares[0])
        cls.instance_id2 = cls._get_instance(cls.shares[1])

    @classmethod
    def _get_instance(cls, share):
        share_instances = cls.admin_client.get_instances_of_share(
            share["id"])['share_instances']
        return share_instances[0]["id"]

    def _verify_create_replica(self, version=LATEST_MICROVERSION):
        # Create the replica
        share_net_id = None
        if utils.is_microversion_ge(version, (
                constants.SHARE_REPLICA_SHARE_NET_PARAM_VERSION)):
            share_net_id = self.sn_id
        share_replica = self.create_share_replica(
            self.shares[0]["id"], self.replica_zone,
            share_network_id=share_net_id, cleanup_in_class=False)
        share_replicas = self.shares_v2_client.list_share_replicas(
            share_id=self.shares[0]["id"])['share_replicas']
        # Ensure replica is created successfully.
        replica_ids = [replica["id"] for replica in share_replicas]
        self.assertIn(share_replica["id"], replica_ids)
        return share_replica

    def _verify_active_replica_count(self, share_id):
        # List replicas
        replica_list = self.shares_v2_client.list_share_replicas(
            share_id=share_id)['share_replicas']

        # Check if there is only 1 'active' replica before promotion.
        active_replicas = self._filter_replica_list(
            replica_list, constants.REPLICATION_STATE_ACTIVE)
        self.assertEqual(1, len(active_replicas))

    def _filter_replica_list(self, replica_list, r_state):
        # Iterate through replica list to filter based on replica_state
        return [replica for replica in replica_list
                if replica['replica_state'] == r_state]

    def _verify_in_sync_replica_promotion(self, share, original_replica):
        # Verify that 'in-sync' replica has been promoted successfully

        # NOTE(Yogi1): Cleanup needs to be disabled for replica that is
        # being promoted since it will become the 'primary'/'active' replica.
        replica = self.create_share_replica(share["id"], self.replica_zone,
                                            cleanup=False)
        # Wait for replica state to update after creation
        waiters.wait_for_resource_status(
            self.shares_v2_client, replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')
        # Promote the first in_sync replica to active state
        promoted_replica = self.promote_share_replica(replica['id'])
        # Delete the demoted replica so promoted replica can be cleaned
        # during the cleanup of the share.
        self.addCleanup(self.delete_share_replica, original_replica['id'])
        self._verify_active_replica_count(share["id"])
        # Verify the replica_state for promoted replica
        promoted_replica = self.shares_v2_client.get_share_replica(
            promoted_replica["id"])['share_replica']
        self.assertEqual(constants.REPLICATION_STATE_ACTIVE,
                         promoted_replica["replica_state"])

    def _check_skip_promotion_tests(self):
        # Check if the replication type is right for replica promotion tests
        if (self.replication_type
                not in constants.REPLICATION_PROMOTION_CHOICES):
            msg = "Option backend_replication_type should be one of (%s)!"
            raise self.skipException(
                msg % ','.join(constants.REPLICATION_PROMOTION_CHOICES))

    @decorators.idempotent_id('c59e3198-062b-4284-8a3b-189a62213573')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(
        CONF.share.multitenancy_enabled, "Only for multitenancy.")
    @ddt.data(
        *utils.deduplicate([constants.SHARE_REPLICA_SHARE_NET_PARAM_VERSION,
                            LATEST_MICROVERSION]))
    def test_create_share_replica_with_provided_network(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        share_replica = self._verify_create_replica(version)
        self.assertIsNotNone(share_replica)

    @decorators.idempotent_id('8858617f-292d-4e5c-9e15-794b7f1b2e3c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_add_delete_share_replica(self):
        # Create the replica
        share_replica = self._verify_create_replica()

        # Delete the replica
        self.delete_share_replica(share_replica["id"])

    @decorators.idempotent_id('58c3faf4-6c97-4fec-9a9b-7cff0d2035cd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    @utils.skip_if_microversion_not_supported("2.51")
    def test_add_delete_share_replica_different_subnet(self):
        # Create new subnet in replica az
        subnet = utils.share_network_get_default_subnet(self.share_network)
        data = {
            'neutron_net_id': subnet.get('neutron_net_id'),
            'neutron_subnet_id': subnet.get('neutron_subnet_id'),
            'share_network_id': self.sn_id,
            'availability_zone': self.replica_zone,
        }
        subnet = self.create_share_network_subnet(**data)
        # Create the replica
        share_replica = self._verify_create_replica()

        # Delete the replica
        self.delete_share_replica(share_replica["id"])
        # Delete subnet
        self.shares_v2_client.delete_subnet(self.sn_id, subnet['id'])

    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    @testtools.skipIf(
        not CONF.share.run_share_server_multiple_subnet_tests,
        "Share server multiple subnet tests are disabled.")
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share networks.")
    @utils.skip_if_microversion_not_supported("2.70")
    @decorators.idempotent_id('4235e789-dbd6-47a8-8f2e-d70edf78e532')
    def test_add_delete_share_replica_multiple_subnets(self):
        extra_specs = {
            "replication_type": self.replication_type,
            "driver_handles_share_servers": CONF.share.multitenancy_enabled,
            "share_server_multiple_subnet_support": True,
        }
        share_type = self.create_share_type(
            extra_specs=extra_specs, client=self.admin_client)
        default_subnet = utils.share_network_get_default_subnet(
            self.share_network)
        new_share_network_id = self.create_share_network(
            cleanup_in_class=False)['id']
        subnet_data = {
            'neutron_net_id': default_subnet.get('neutron_net_id'),
            'neutron_subnet_id': default_subnet.get('neutron_subnet_id'),
            'share_network_id': new_share_network_id,
            'availability_zone': self.replica_zone,
        }
        subnet1 = self.create_share_network_subnet(**subnet_data)
        subnet2 = self.create_share_network_subnet(**subnet_data)
        # Creating a third subnet in share replica az
        subnet_data.update({'availability_zone': self.share_zone})
        subnet3 = self.create_share_network_subnet(**subnet_data)
        # Create the share and share replica
        share = self.create_share(
            share_type_id=share_type['id'], cleanup_in_class=False,
            availability_zone=self.share_zone,
            share_network_id=new_share_network_id)
        share = self.admin_client.get_share(share['id'])['share']
        replica = self.create_share_replica(share['id'], self.replica_zone)
        replica = self.admin_client.get_share_replica(
            replica['id'])['share_replica']
        share_server = self.admin_client.show_share_server(
            replica['share_server_id'])['share_server']
        self.assertIn(subnet1['id'],
                      share_server['share_network_subnet_ids'])
        self.assertIn(subnet2['id'],
                      share_server['share_network_subnet_ids'])
        # Delete the replica
        self.delete_share_replica(replica['id'])
        # Delete share
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
        # Delete subnets
        self.shares_v2_client.delete_subnet(
            new_share_network_id, subnet1['id'])
        self.shares_v2_client.delete_subnet(
            new_share_network_id, subnet2['id'])
        self.shares_v2_client.delete_subnet(
            new_share_network_id, subnet3['id'])

    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    @testtools.skipIf(
        not CONF.share.run_network_allocation_update_tests,
        "Share server network allocation update tests are disabled.")
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    @utils.skip_if_microversion_not_supported("2.70")
    @decorators.idempotent_id('26694947-d4a0-46c8-99e8-2e0eca1b6a08')
    def test_add_delete_share_replica_network_allocation_update(self):
        extra_specs = {
            "replication_type": self.replication_type,
            "driver_handles_share_servers": CONF.share.multitenancy_enabled,
            "network_allocation_update_support": True,
        }
        share_type = self.create_share_type(extra_specs=extra_specs)

        default_subnet = utils.share_network_get_default_subnet(
            self.share_network)
        new_share_network_id = self.create_share_network(
            cleanup_in_class=False)['id']
        subnet_data = {
            'neutron_net_id': default_subnet.get('neutron_net_id'),
            'neutron_subnet_id': default_subnet.get('neutron_subnet_id'),
            'share_network_id': new_share_network_id,
            'availability_zone': self.share_zone,
        }
        subnet1 = self.create_share_network_subnet(**subnet_data)
        subnet_data.update({'availability_zone': self.replica_zone})
        subnet2 = self.create_share_network_subnet(**subnet_data)
        # Create the share and share replica
        share = self.create_share(
            share_type_id=share_type['id'], cleanup_in_class=False,
            availability_zone=self.share_zone,
            share_network_id=new_share_network_id)
        share = self.admin_client.get_share(share['id'])['share']

        replica = self.create_share_replica(share['id'], self.replica_zone)
        replica = self.admin_client.get_share_replica(
            replica['id'])['share_replica']

        # Waits until the check is completed and positive
        waiters.wait_for_subnet_create_check(
            self.shares_v2_client, new_share_network_id,
            neutron_net_id=subnet_data['neutron_net_id'],
            neutron_subnet_id=subnet_data['neutron_subnet_id'],
            availability_zone=self.replica_zone)
        # Creating a third subnet in replica zone to trigger the network
        # allocation update
        subnet3 = self.create_share_network_subnet(**subnet_data)
        waiters.wait_for_resource_status(
            self.admin_client, replica['share_server_id'],
            constants.SERVER_STATE_ACTIVE,
            resource_name="share_server",
            status_attr="status")
        share_server = self.admin_client.show_share_server(
            replica['share_server_id']
        )['share_server']
        self.assertIn(subnet2['id'],
                      share_server['share_network_subnet_ids'])
        self.assertIn(subnet3['id'],
                      share_server['share_network_subnet_ids'])
        # Delete the replica
        self.delete_share_replica(replica['id'])
        # Delete share
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
        # Delete subnets
        self.shares_v2_client.delete_subnet(
            new_share_network_id, subnet1['id'])
        self.shares_v2_client.delete_subnet(
            new_share_network_id, subnet2['id'])
        self.shares_v2_client.delete_subnet(
            new_share_network_id, subnet3['id'])

    @decorators.idempotent_id('00e12b41-b95d-494a-99be-e584aae10f5c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_add_access_rule_create_replica_delete_rule(self):
        # Add access rule to the share
        access_type, access_to = utils.get_access_rule_data_from_config(
            self.shares_v2_client.share_protocol)
        self.allow_access(
            self.shares[0]["id"], access_type=access_type, access_to=access_to,
            access_level='ro')

        # Create the replica
        self._verify_create_replica()

        # Verify access_rules_status transitions to 'active' state.
        waiters.wait_for_resource_status(
            self.shares_v2_client, self.shares[0]["id"],
            constants.RULE_STATE_ACTIVE, status_attr='access_rules_status')

    @decorators.idempotent_id('3af3f19a-1195-464e-870b-1a3918914f1b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_create_replica_add_access_rule_delete_replica(self):
        access_type, access_to = utils.get_access_rule_data_from_config(
            self.shares_v2_client.share_protocol)
        # Create the replica
        share_replica = self._verify_create_replica()

        # Add access rule
        self.allow_access(
            self.shares[0]["id"], access_type=access_type, access_to=access_to,
            access_level='ro')

        # Delete the replica
        self.delete_share_replica(share_replica["id"])

    @decorators.idempotent_id('600a13d2-5cf0-482e-97af-9f598b55a406')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.74")
    def test_add_access_rule_share_replica_error_status(self):
        '''From 2.74, we can add rules even if replicas are in error state.'''
        access_type, access_to = utils.get_access_rule_data_from_config(
            self.shares_v2_client.share_protocol)
        # Create the replica
        share_replica = self._verify_create_replica()

        # Reset the replica status to error
        self.admin_client.reset_share_replica_status(
            share_replica['id'], constants.STATUS_ERROR)

        # Verify access rule will be added in error state
        self.shares_v2_client.create_access_rule(
            self.shares[0]["id"], access_type=access_type, access_to=access_to,
            access_level='ro')

        # Verify access_rules_status transitions to 'active' state.
        waiters.wait_for_resource_status(
            self.shares_v2_client, self.shares[0]["id"],
            constants.RULE_STATE_ACTIVE, status_attr='access_rules_status')

    @decorators.idempotent_id('a542c179-ea41-4bc0-bd80-e06eaddf5253')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(CONF.share.run_multiple_share_replicas_tests,
                          'Multiple share replicas tests are disabled.')
    def test_add_multiple_share_replicas(self):
        rep_domain, pools = self.get_pools_for_replication_domain()
        if len(pools) < 3:
            msg = ("Replication domain %(domain)s has only %(count)s pools. "
                   "Need at least 3 pools to run this test." %
                   {"domain": rep_domain, "count": len(pools)})
            raise self.skipException(msg)
        # Add the replicas
        share_replica1 = self.create_share_replica(self.shares[0]["id"],
                                                   self.replica_zone,
                                                   cleanup_in_class=False)
        share_replica2 = self.create_share_replica(self.shares[0]["id"],
                                                   self.replica_zone,
                                                   cleanup_in_class=False)
        self.shares_v2_client.get_share_replica(share_replica2['id'])

        share_replicas = self.admin_client.list_share_replicas(
            share_id=self.shares[0]["id"])['share_replicas']
        replica_host_set = {r['host'] for r in share_replicas}

        # Assert that replicas are created on different pools.
        msg = "More than one replica is created on the same pool."
        self.assertEqual(3, len(replica_host_set), msg)
        # Verify replicas are in the replica list
        replica_ids = [replica["id"] for replica in share_replicas]
        self.assertIn(share_replica1["id"], replica_ids)
        self.assertIn(share_replica2["id"], replica_ids)

    @decorators.idempotent_id('98b7c1d6-02e8-425a-b697-db2d2671fa11')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_promote_in_sync_share_replica(self):
        # Test promote 'in_sync' share_replica to 'active' state
        self._check_skip_promotion_tests()
        share = self.create_shares([self.creation_data])[0]
        original_replica = self.shares_v2_client.list_share_replicas(
            share["id"])['share_replicas'][0]
        self._verify_in_sync_replica_promotion(share, original_replica)

    @decorators.idempotent_id('3af912f4-b5d7-4241-b2b3-bdf12ff398a4')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_add_rule_promote_share_replica_verify_rule(self):
        # Verify the access rule stays intact after share replica promotion
        self._check_skip_promotion_tests()

        share = self.create_shares([self.creation_data])[0]
        # Add access rule
        access_type, access_to = utils.get_access_rule_data_from_config(
            self.shares_v2_client.share_protocol)
        self.allow_access(
            share["id"], access_type=access_type, access_to=access_to,
            access_level='ro')

        original_replica = self.shares_v2_client.list_share_replicas(
            share["id"])['share_replicas'][0]
        self._verify_in_sync_replica_promotion(share, original_replica)

        # verify rule's values
        rules_list = self.shares_v2_client.list_access_rules(
            share["id"])['access_list']
        self.assertEqual(1, len(rules_list))
        self.assertEqual(access_type, rules_list[0]["access_type"])
        self.assertEqual(access_to, rules_list[0]["access_to"])
        self.assertEqual('ro', rules_list[0]["access_level"])

    @decorators.idempotent_id('7904e3c7-e6d0-472d-b9c9-c0772b4f9f1b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.48")
    def test_share_type_azs_share_replicas(self):
        az_spec = ', '.join(self.zones)
        self.admin_shares_v2_client.update_share_type_extra_spec(
            self.share_type['id'], 'availability_zones', az_spec)
        self.addCleanup(
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            self.share_type['id'], 'availability_zones')

        share = self.create_share(
            share_type_id=self.share_type['id'], cleanup_in_class=False,
            availability_zone=self.share_zone, share_network_id=self.sn_id)
        share = self.shares_v2_client.get_share(share['id'])['share']
        replica = self.create_share_replica(share['id'], self.replica_zone)
        replica = self.shares_v2_client.get_share_replica(
            replica['id'])['share_replica']

        self.assertEqual(self.share_zone, share['availability_zone'])
        self.assertEqual(self.replica_zone, replica['availability_zone'])
        self.assertIn(share['availability_zone'], self.zones)
        self.assertIn(replica['availability_zone'], self.zones)

    @decorators.idempotent_id('b5ade58b-cb81-47eb-966b-28e6d85b5568')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_promote_and_promote_back(self):
        # Test promote back and forth between 2 share replicas
        self._check_skip_promotion_tests()

        # Create a new share
        share = self.create_shares([self.creation_data])[0]

        # Discover the original replica
        initial_replicas = self.shares_v2_client.list_share_replicas(
            share_id=share['id'])['share_replicas']
        self.assertEqual(1, len(initial_replicas),
                         '%s replicas initially created for share %s' %
                         (len(initial_replicas), share['id']))
        original_replica = initial_replicas[0]

        # Create a new replica
        new_replica = self.create_share_replica(share["id"],
                                                self.replica_zone,
                                                cleanup_in_class=False)
        waiters.wait_for_resource_status(
            self.shares_v2_client, new_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')

        # Promote the new replica to active and verify the replica states
        self.promote_share_replica(new_replica['id'])
        self._verify_active_replica_count(share["id"])
        waiters.wait_for_resource_status(
            self.shares_v2_client, original_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')

        # Promote the original replica back to active
        self.promote_share_replica(original_replica['id'])
        self._verify_active_replica_count(share["id"])
        waiters.wait_for_resource_status(
            self.shares_v2_client, new_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')

    @decorators.idempotent_id('1452156b-75a5-4f3c-a921-834732a03b0a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_active_replication_state(self):
        # Verify the replica_state of first instance is set to active.
        replica = self.shares_v2_client.get_share_replica(
            self.instance_id1)['share_replica']
        self.assertEqual(
            constants.REPLICATION_STATE_ACTIVE, replica['replica_state'])


class ReplicationActionsTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ReplicationActionsTest, cls).skip_checks()
        if not CONF.share.run_replication_tests:
            raise cls.skipException('Replication tests are disabled.')

        utils.check_skip_if_microversion_not_supported(
            _MIN_SUPPORTED_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ReplicationActionsTest, cls).resource_setup()
        # Create share_type
        name = data_utils.rand_name(constants.TEMPEST_MANILA_PREFIX)
        cls.admin_client = cls.admin_shares_v2_client
        cls.replication_type = CONF.share.backend_replication_type
        cls.multitenancy_enabled = (
            utils.replication_with_multitenancy_support())

        if cls.replication_type not in constants.REPLICATION_TYPE_CHOICES:
            raise share_exceptions.ShareReplicationTypeException(
                replication_type=cls.replication_type
            )
        cls.extra_specs = cls.add_extra_specs_to_dict(
            {"replication_type": cls.replication_type})
        cls.share_type = cls.create_share_type(
            name,
            extra_specs=cls.extra_specs,
            client=cls.admin_client)

        cls.zones = cls.get_availability_zones_matching_share_type(
            cls.share_type, client=cls.admin_client)
        cls.share_zone = cls.zones[0]
        cls.replica_zone = cls.zones[-1]

        # Create share with above share_type
        cls.creation_data = {'kwargs': {
            'share_type_id': cls.share_type['id'],
            'availability_zone': cls.share_zone,
        }}

        if cls.multitenancy_enabled:
            cls.share_network = cls.shares_v2_client.get_share_network(
                cls.shares_v2_client.share_network_id)['share_network']
            cls.creation_data['kwargs'].update({
                'share_network_id': cls.share_network['id']})
        cls.sn_id = (
            cls.share_network['id'] if cls.multitenancy_enabled else None)
        # Data for creating shares in parallel
        data = [cls.creation_data, cls.creation_data]
        cls.shares = cls.create_shares(data)
        cls.shares = [cls.shares_v2_client.get_share(s['id'])['share'] for s in
                      cls.shares]
        cls.instance_id1 = cls._get_instance(cls.shares[0])
        cls.instance_id2 = cls._get_instance(cls.shares[1])

        # Create replicas to 2 shares
        cls.replica1 = cls.create_share_replica(cls.shares[0]["id"],
                                                cls.replica_zone,
                                                cleanup_in_class=True)
        cls.replica2 = cls.create_share_replica(cls.shares[1]["id"],
                                                cls.replica_zone,
                                                cleanup_in_class=True)

    @classmethod
    def _get_instance(cls, share):
        share_instances = cls.admin_client.get_instances_of_share(
            share["id"])['share_instances']
        return share_instances[0]["id"]

    def _validate_replica_list(self, replica_list, detail=True):
        # Verify keys
        if detail:
            keys = DETAIL_KEYS
        else:
            keys = SUMMARY_KEYS
        for replica in replica_list:
            self.assertEqual(sorted(keys), sorted(replica.keys()))
            # Check for duplicates
            replica_id_list = [sr["id"] for sr in replica_list
                               if sr["id"] == replica["id"]]
            msg = "Replica %s appears %s times in replica list." % (
                replica['id'], len(replica_id_list))
            self.assertEqual(1, len(replica_id_list), msg)

    @decorators.idempotent_id('abe0e49d-0b94-4b81-a220-ab047712492d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_show_share_replica(self):
        replica = self.shares_v2_client.get_share_replica(
            self.replica1["id"])['share_replica']

        actual_keys = sorted(list(replica.keys()))
        detail_keys = sorted(DETAIL_KEYS)
        self.assertEqual(detail_keys, actual_keys,
                         'Share Replica %s has incorrect keys; '
                         'expected %s, got %s.' % (replica["id"],
                                                   detail_keys, actual_keys))

    @decorators.idempotent_id('f5225fb7-fcbe-4825-bf5b-0e11c2d26e03')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_detail_list_share_replicas_for_share(self):
        # List replicas for share
        replica_list = self.shares_v2_client.list_share_replicas(
            share_id=self.shares[0]["id"])['share_replicas']
        replica_ids_list = [rep['id'] for rep in replica_list]
        self.assertIn(self.replica1['id'], replica_ids_list,
                      'Replica %s was not returned in the list of replicas: %s'
                      % (self.replica1['id'], replica_list))
        # Verify keys
        self._validate_replica_list(replica_list)

    @decorators.idempotent_id('e39aeb5d-fe4b-4896-8615-e6e7290bcb56')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_detail_list_share_replicas_for_all_shares(self):
        # List replicas for all available shares
        replica_list = self.shares_v2_client.list_share_replicas(
            )['share_replicas']
        replica_ids_list = [rep['id'] for rep in replica_list]
        for replica in [self.replica1, self.replica2]:
            self.assertIn(replica['id'], replica_ids_list,
                          'Replica %s was not returned in the list of '
                          'replicas: %s' % (replica['id'], replica_list))
        # Verify keys
        self._validate_replica_list(replica_list)

    @decorators.idempotent_id('8d11848a-7766-41e5-af09-6121e5bad447')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_summary_list_share_replicas_for_all_shares(self):
        # List replicas
        replica_list = self.shares_v2_client.list_share_replicas_summary(
            )['share_replicas']

        # Verify keys
        self._validate_replica_list(replica_list, detail=False)



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacShareTypesTests(rbac_base.ShareRbacBaseTests,
                               metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(ShareRbacShareTypesTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()

    @classmethod
    def resource_setup(cls):
        super(ShareRbacShareTypesTests, cls).resource_setup()
        cls.share_type = cls.get_share_type()

    def share_type_properties(self):
        share_type = {}
        share_type['extra_specs'] = {
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }
        share_type['name'] = data_utils.rand_name("share-type")
        return share_type

    @abc.abstractmethod
    def test_create_share_type(self):
        pass

    @abc.abstractmethod
    def test_get_share_type(self):
        pass

    @abc.abstractmethod
    def test_list_share_type(self):
        pass

    @abc.abstractmethod
    def test_update_share_type(self):
        pass


class ProjectAdminTests(ShareRbacShareTypesTests, base.BaseSharesTest):

    credentials = ['project_admin']

    @classmethod
    def setup_clients(cls):
        super(ProjectAdminTests, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.persona, project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('b24bf137-352a-4ebd-b736-27518d32c1bd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_share_type(self):
        share_type = self.do_request(
            'create_share_type', expected_status=200,
            **self.share_type_properties())['share_type']
        self.addCleanup(self.delete_resource, self.client,
                        st_id=share_type['id'])

    @decorators.idempotent_id('741d69f3-b3fe-49cf-9e33-6b0696b353ec')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_share_type(self):
        self.do_request(
            'get_share_type', expected_status=200,
            share_type_id=self.share_type['id'])

    @decorators.idempotent_id('3f811ac6-a345-424f-863a-1a7a49ba0a32')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_type(self):
        share_type_list = self.do_request(
            'list_share_types', expected_status=200)['share_types']
        share_type_id_list = [
            st['id'] for st in share_type_list
        ]
        self.assertIn(self.share_type['id'], share_type_id_list)

    @decorators.idempotent_id('3bb9aaab-3c17-45be-a9b1-dd8b6942cb59')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_share_type(self):
        share_type = self.client.create_share_type(
            **self.share_type_properties())['share_type']
        self.addCleanup(self.client.delete_share_type, share_type['id'])

        name = data_utils.rand_name("updated_share_type")
        self.do_request(
            'update_share_type', expected_status=200,
            share_type_id=share_type['id'], name=name)


class ProjectMemberTests(ShareRbacShareTypesTests, base.BaseSharesTest):

    credentials = ['project_member']

    @decorators.idempotent_id('270761cf-07b4-4fc7-96b5-4deb205adce3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_type(self):
        self.do_request(
            'create_share_type', expected_status=lib_exc.Forbidden,
            **self.share_type_properties())

    @decorators.idempotent_id('d3f53218-d92f-489d-8e2e-985178e7fd02')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_share_type(self):
        self.do_request(
            'get_share_type', expected_status=200,
            share_type_id=self.share_type['id'])

    @decorators.idempotent_id('757c7ccd-e14e-4c1a-9172-998ae5eed1b8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_type(self):
        share_type_list = self.do_request(
            'list_share_types', expected_status=200)['share_types']
        share_type_id_list = [
            st['id'] for st in share_type_list
        ]
        self.assertIn(self.share_type['id'], share_type_id_list)

    @decorators.idempotent_id('5210170c-b749-4645-a86f-7347c3ba3e99')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_update_share_type(self):
        name = data_utils.rand_name("updated_share_type")
        self.do_request(
            'update_share_type', expected_status=lib_exc.Forbidden,
            share_type_id=self.share_type['id'], name=name)


class ProjectReaderTests(ShareRbacShareTypesTests, base.BaseSharesTest):

    credentials = ['project_reader']

    @decorators.idempotent_id('f4c352c4-c12b-4722-9fe7-9a2ec639ee63')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_type(self):
        self.do_request(
            'create_share_type', expected_status=lib_exc.Forbidden,
            **self.share_type_properties())

    @decorators.idempotent_id('e9d9f244-7778-443b-aadc-bac9f2b687b7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_share_type(self):
        self.do_request(
            'get_share_type', expected_status=200,
            share_type_id=self.share_type['id'])

    @decorators.idempotent_id('cf0e97f1-4853-4cf2-9e9a-041c6e57bab5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_type(self):
        share_type_list = self.do_request(
            'list_share_types', expected_status=200)['share_types']
        share_type_id_list = [
            st['id'] for st in share_type_list
        ]
        self.assertIn(self.share_type['id'], share_type_id_list)

    @decorators.idempotent_id('338d579b-ff91-4a30-af53-d0b317919efb')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_update_share_type(self):
        name = data_utils.rand_name("updated_share_type")
        self.do_request(
            'update_share_type', expected_status=lib_exc.Forbidden,
            share_type_id=self.share_type['id'], name=name)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base


@ddt.ddt
class ServicesAdminTest(base.BaseSharesAdminTest):

    def setUp(self):
        super(ServicesAdminTest, self).setUp()
        self.services = self.shares_client.list_services()['services']

    @decorators.idempotent_id('74cd12ab-a1f5-40fb-9110-d9035b4b20c5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_list_services(self, client_name):
        services = getattr(self, client_name).list_services()['services']
        self.assertNotEqual(0, len(services))

        for service in services:
            self.assertIsNotNone(service['id'])

    @decorators.idempotent_id('e80f4b89-7280-47eb-9cbf-01a09e04c2d8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_get_services_by_host_name(self, client_name):
        host = self.services[0]["host"]
        params = {"host": host}
        services = getattr(self, client_name).list_services(params)['services']
        self.assertNotEqual(0, len(services))
        for service in services:
            self.assertEqual(host, service["host"])

    @decorators.idempotent_id('1dd4d799-b900-4476-9e51-ad9db6ee4435')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_get_services_by_binary_name(self, client_name):
        binary = self.services[0]["binary"]
        params = {"binary": binary, }
        services = getattr(self, client_name).list_services(params)['services']
        self.assertNotEqual(0, len(services))
        for service in services:
            self.assertEqual(binary, service["binary"])

    @decorators.idempotent_id('d12ea678-025f-46b4-95c5-3a03b3e440d7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_get_services_by_availability_zone(self, client_name):
        zone = self.services[0]["zone"]
        params = {"zone": zone, }
        services = getattr(self, client_name).list_services(params)['services']
        self.assertNotEqual(0, len(services))
        for service in services:
            self.assertEqual(zone, service["zone"])

    @decorators.idempotent_id('e82921a7-6c98-4c9c-a47b-34a0badc3b59')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_get_services_by_status(self, client_name):
        status = self.services[0]["status"]
        params = {"status": status, }
        services = getattr(self, client_name).list_services(params)['services']
        self.assertNotEqual(0, len(services))
        for service in services:
            self.assertEqual(status, service["status"])

    @decorators.idempotent_id('c30234f0-1331-4560-93a7-cbda7d00eb53')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_get_services_by_state(self, client_name):
        state = self.services[0]["state"]
        params = {"state": state, }
        services = getattr(self, client_name).list_services(params)['services']
        self.assertNotEqual(0, len(services))
        for service in services:
            self.assertEqual(state, service["state"])

    @decorators.idempotent_id('b77c4bc4-57d5-4181-9e95-e230ab682b32')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('shares_client', 'shares_v2_client')
    def test_get_services_by_all_filters(self, client_name):
        params = {
            "host": self.services[0]["host"],
            "binary": self.services[0]["binary"],
            "zone": self.services[0]["zone"],
            "status": self.services[0]["status"],
            "state": self.services[0]["state"],
        }
        services = getattr(self, client_name).list_services(params)['services']
        self.assertNotEqual(0, len(services))
        for service in services:
            self.assertEqual(params["host"], service["host"])
            self.assertEqual(params["binary"], service["binary"])
            self.assertEqual(params["zone"], service["zone"])
            self.assertEqual(params["status"], service["status"])
            self.assertEqual(params["state"], service["state"])



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base


CONF = config.CONF


class ShareMultiBackendTest(base.BaseSharesAdminTest):

    @staticmethod
    def _share_protocol(protocol):
        protocols_list = protocol.lower().split('_')
        allowed_protocols = [
            i for i in CONF.share.enable_protocols
            if i.lower() in protocols_list
        ]
        return allowed_protocols

    @classmethod
    def resource_setup(cls):
        super(ShareMultiBackendTest, cls).resource_setup()
        if not CONF.share.multi_backend:
            raise cls.skipException("Manila multi-backend tests are disabled.")
        elif len(CONF.share.backend_names) < 2:
            raise cls.skipException("For running multi-backend tests required"
                                    " two names in config. Skipping.")
        elif any(not name for name in CONF.share.backend_names):
            raise cls.skipException("Share backend names can not be empty. "
                                    "Skipping.")
        cls.sts = []
        cls.shares = []
        share_data_list = []

        pools = cls.shares_v2_client.list_pools(detail=True)['pools']
        backends_protocols = {
            pool['capabilities']['share_backend_name']: pool[
                'capabilities']['storage_protocol'] for pool in pools
        }
        # Create share types
        for backend in CONF.share.backend_names:
            share_protocol = cls._share_protocol(backends_protocols[backend])
            if not share_protocol:
                continue
            st_name = data_utils.rand_name(
                cls.__name__ + "-share-type-%s" % backend)
            extra_specs = {
                "share_backend_name": backend,
                "driver_handles_share_servers":
                    CONF.share.multitenancy_enabled,
            }
            st = cls.create_share_type(name=st_name, extra_specs=extra_specs)
            cls.sts.append(st)
            st_id = st["id"]
            share_data_list.append({"kwargs": {
                "share_type_id": st_id,
                "share_protocol": share_protocol[0]}})

        if not share_data_list:
            raise cls.skipException("Enabled protocols not supported by any "
                                    "of the enabled backends.")

        # Create shares using precreated share types
        cls.shares = cls.create_shares(share_data_list)

    @decorators.idempotent_id('888add9d-4ad0-4d88-a62f-af7ee1d9f9ab')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_share_backend_name_reporting(self):
        # Share's 'host' should be like "hostname@backend_name"
        for share in self.shares:
            get = self.shares_client.get_share(share['id'])['share']
            self.assertEqual(2, len(get["host"].split("@")))

    @decorators.idempotent_id('691fbcef-6d8d-4ad9-b493-501bbb3dcf3c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_share_share_type(self):
        # Share type should be the same as provided with share creation
        for share, share_type in zip(self.shares, self.sts):
            share_details = self.shares_v2_client.get_share(
                share['id'], version="2.5")['share']
            self.assertEqual(share_type["name"], share_details["share_type"])

    @decorators.idempotent_id('f25e0cb0-d656-4f16-a761-ec23992cd9e7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_share_share_type_v_2_6(self):
        # Share type should be the same as provided with share creation
        for share, share_type in zip(self.shares, self.sts):
            share_details = self.shares_v2_client.get_share(
                share['id'], version="2.6")['share']
            self.assertEqual(share_type["id"], share_details["share_type"])
            self.assertEqual(
                share_type["name"], share_details["share_type_name"])

    @decorators.idempotent_id('bfa0c056-0a15-40e1-bdff-f1e10b95736c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_share_backend_name_distinction(self):
        # Different share backends should have different host records
        if CONF.share.backend_names[0] == CONF.share.backend_names[1]:
            raise self.skipException("Share backends "
                                     "configured with same name. Skipping.")
        get1 = self.shares_client.get_share(self.shares[0]['id'])['share']
        get2 = self.shares_client.get_share(self.shares[1]['id'])['share']
        self.assertNotEqual(get1["host"], get2["host"])



# Copyright 2014 Mirantis Inc.  All Rights Reserved.
# Copyright (c) 2015 Yogesh Kshirsagar.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class SharesActionsAdminTest(base.BaseSharesAdminTest):
    """Covers share functionality, that doesn't related to share type."""

    @classmethod
    def resource_setup(cls):
        super(SharesActionsAdminTest, cls).resource_setup()

        cls.shares = []

        # create share type for share filtering purposes
        specs = {"storage_protocol": CONF.share.capability_storage_protocol}
        if CONF.share.capability_snapshot_support:
            specs.update({'snapshot_support': True})
        if CONF.share.capability_create_share_from_snapshot_support:
            specs.update({'create_share_from_snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=specs)
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share_name = data_utils.rand_name("tempest-share-name")
        cls.share_desc = data_utils.rand_name("tempest-share-description")
        cls.metadata = {
            'foo_key_share_1': 'foo_value_share_1',
            'bar_key_share_1': 'foo_value_share_1',
        }
        cls.shares.append(cls.create_share(
            name=cls.share_name,
            description=cls.share_desc,
            metadata=cls.metadata,
            share_type_id=cls.share_type_id,
        ))

        if CONF.share.run_snapshot_tests:
            # create snapshot
            cls.snap_name = data_utils.rand_name("tempest-snapshot-name")
            cls.snap_desc = data_utils.rand_name(
                "tempest-snapshot-description")
            cls.snap = cls.create_snapshot_wait_for_active(
                cls.shares[0]["id"], cls.snap_name, cls.snap_desc)

            if CONF.share.capability_create_share_from_snapshot_support:

                # create second share from snapshot for purposes of sorting and
                # snapshot filtering
                cls.share_name2 = data_utils.rand_name("tempest-share-name")
                cls.share_desc2 = data_utils.rand_name(
                    "tempest-share-description")
                cls.metadata2 = {
                    'foo_key_share_2': 'foo_value_share_2',
                    'bar_key_share_2': 'foo_value_share_2',
                }
                cls.shares.append(cls.create_share(
                    name=cls.share_name2,
                    description=cls.share_desc2,
                    metadata=cls.metadata2,
                    snapshot_id=cls.snap['id'],
                    share_type_id=cls.share_type_id,
                ))

    @decorators.idempotent_id('5f61f5dd-891e-478f-b102-803096820882')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share(self):

        # get share
        share = self.shares_client.get_share(self.shares[0]['id'])['share']

        # verify keys
        expected_keys = ["status", "description", "links", "availability_zone",
                         "created_at", "export_location", "share_proto",
                         "name", "snapshot_id", "id", "size"]
        actual_keys = share.keys()
        [self.assertIn(key, actual_keys) for key in expected_keys]

        # verify values
        msg = "Expected name: '%s', actual name: '%s'" % (self.share_name,
                                                          share["name"])
        self.assertEqual(self.share_name, str(share["name"]), msg)

        msg = ("Expected description: '%s', "
               "actual description: '%s'" % (self.share_desc,
                                             share["description"]))
        self.assertEqual(self.share_desc, str(share["description"]), msg)

        msg = "Expected size: '%s', actual size: '%s'" % (
            CONF.share.share_size, share["size"])
        self.assertEqual(CONF.share.share_size, int(share["size"]), msg)

    @decorators.idempotent_id('60d34573-8452-47ab-9455-0067bdd3ed9c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares(self):

        # list shares
        shares = self.shares_client.list_shares()['shares']

        # verify keys
        keys = ["name", "id", "links"]
        [self.assertIn(key, sh.keys()) for sh in shares for key in keys]

        # our share id in list and have no duplicates
        for share in self.shares:
            gen = [sid["id"] for sid in shares if sid["id"] in share["id"]]
            msg = "expected id lists %s times in share list" % (len(gen))
            self.assertEqual(1, len(gen), msg)

    @decorators.idempotent_id('85f9438d-d3f6-4f3a-8134-e89915373df3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail(self):

        # list shares
        shares = self.shares_client.list_shares_with_detail()['shares']

        # verify keys
        keys = [
            "status", "description", "links", "availability_zone",
            "created_at", "export_location", "share_proto", "host",
            "name", "snapshot_id", "id", "size", "project_id",
        ]
        [self.assertIn(key, sh.keys()) for sh in shares for key in keys]

        # our shares in list and have no duplicates
        for share in self.shares:
            gen = [sid["id"] for sid in shares if sid["id"] in share["id"]]
            msg = "expected id lists %s times in share list" % (len(gen))
            self.assertEqual(1, len(gen), msg)

    @decorators.idempotent_id('47dad08b-0c36-428f-8ab9-5eba92ffc995')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_metadata(self):
        filters = {'metadata': self.metadata}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        for share in shares:
            self.assertLessEqual(filters['metadata'].items(),
                                 share['metadata'].items())
        if CONF.share.capability_create_share_from_snapshot_support:
            self.assertFalse(self.shares[1]['id'] in [s['id'] for s in shares])

    @decorators.idempotent_id('d884c91e-88f5-4e42-83d9-ec3b440af893')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_extra_specs(self):
        filters = {
            "extra_specs": {
                "storage_protocol": CONF.share.capability_storage_protocol,
            }
        }
        share_type_list = self.shares_client.list_share_types()["share_types"]

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        shares_ids = [s["id"] for s in shares]
        for share in self.shares:
            self.assertIn(share["id"], shares_ids)
        for share in shares:
            # find its name or id, get id
            st_id = None
            for st in share_type_list:
                if share["share_type"] in (st["id"], st["name"]):
                    st_id = st["id"]
                    break
            if st_id is None:
                raise ValueError(
                    "Share '%(s_id)s' listed with extra_specs filter has "
                    "nonexistent share type '%(st)s'." % {
                        "s_id": share["id"], "st": share["share_type"]}
                )
            extra_specs = self.shares_client.get_share_type_extra_specs(
                st_id)['extra_specs']
            self.assertLessEqual(filters["extra_specs"].items(),
                                 extra_specs.items())

    @decorators.idempotent_id('76fbe8ba-f1d3-4446-b9b8-55617762a2c7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_share_type_id(self):
        filters = {'share_type_id': self.share_type_id}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        for share in shares:
            st_list = self.shares_client.list_share_types()
            # find its name or id, get id
            sts = st_list["share_types"]
            st_id = None
            for st in sts:
                if share["share_type"] in [st["id"], st["name"]]:
                    st_id = st["id"]
                    break
            if st_id is None:
                raise ValueError(
                    "Share '%(s_id)s' listed with share_type_id filter has "
                    "nonexistent share type '%(st)s'." % {
                        "s_id": share["id"], "st": share["share_type"]}
                )
            self.assertEqual(
                filters['share_type_id'], st_id)
        share_ids = [share['id'] for share in shares]
        for share in self.shares:
            self.assertIn(share['id'], share_ids)

    @decorators.idempotent_id('04afc330-78ee-494f-a660-7670c877a440')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_host(self):
        base_share = self.shares_client.get_share(
            self.shares[0]['id'])['share']
        filters = {'host': base_share['host']}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        for share in shares:
            self.assertEqual(filters['host'], share['host'])

    @utils.skip_if_microversion_not_supported("2.35")
    @ddt.data(('path', True), ('id', True), ('path', False), ('id', False))
    @ddt.unpack
    @decorators.idempotent_id('a27e5e3f-451f-4200-af38-99a562ccbe86')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_or_with_detail_filter_by_export_location(
            self, export_location_type, enable_detail):
        export_locations = self.shares_v2_client.list_share_export_locations(
            self.shares[0]['id'])['export_locations']
        if not isinstance(export_locations, (list, tuple, set)):
            export_locations = (export_locations, )

        filters = {
            'export_location_' + export_location_type:
                export_locations[0][export_location_type],
        }
        # list shares
        if enable_detail:
            shares = self.shares_v2_client.list_shares_with_detail(
                params=filters)['shares']
        else:
            shares = self.shares_v2_client.list_shares(
                params=filters)['shares']

        # verify response
        self.assertEqual(1, len(shares))
        self.assertEqual(self.shares[0]['id'], shares[0]['id'])

    @decorators.idempotent_id('4582de51-1dcd-4c44-b550-eca9a9685038')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    def test_list_shares_with_detail_filter_by_share_network_id(self):
        base_share = self.shares_client.get_share(
            self.shares[0]['id'])['share']
        filters = {'share_network_id': base_share['share_network_id']}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        for share in shares:
            self.assertEqual(
                filters['share_network_id'], share['share_network_id'])

    @decorators.idempotent_id('645aebc4-55ac-406d-b7ab-5614c4fc12e6')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Create share from snapshot tests are disabled.")
    def test_list_shares_with_detail_filter_by_snapshot_id(self):
        filters = {'snapshot_id': self.snap['id']}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        for share in shares:
            self.assertEqual(filters['snapshot_id'], share['snapshot_id'])
        self.assertFalse(self.shares[0]['id'] in [s['id'] for s in shares])

    @decorators.idempotent_id('87659cee-4692-412a-9bfe-06fc97d30ba0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_with_asc_sorting(self):
        filters = {'sort_key': 'created_at', 'sort_dir': 'asc'}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        sorted_list = [share['created_at'] for share in shares]
        self.assertEqual(sorted(sorted_list), sorted_list)

    @decorators.idempotent_id('631f4226-f1ea-47b1-a472-8f12da2d05c4')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_existed_name(self):
        # list shares by name, at least one share is expected
        params = {"name": self.share_name}
        shares = self.shares_client.list_shares_with_detail(params)['shares']
        self.assertEqual(self.share_name, shares[0]["name"])

    @decorators.idempotent_id('d0dae9e5-a826-48e4-b7b7-24b08ad5a7cb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_fake_name(self):
        # list shares by fake name, no shares are expected
        params = {"name": data_utils.rand_name("fake-nonexistent-name")}
        shares = self.shares_client.list_shares_with_detail(params)['shares']
        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('8eac9b63-666f-4c52-8c5f-58b1fdf201e2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_active_status(self):
        # list shares by active status, at least one share is expected
        params = {"status": "available"}
        shares = self.shares_client.list_shares_with_detail(params)['shares']
        self.assertGreater(len(shares), 0)
        for share in shares:
            self.assertEqual(params["status"], share["status"])

    @decorators.idempotent_id('e94f41c0-f6c4-4d77-b4f9-2c796c27e348')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_fake_status(self):
        # list shares by fake status, no shares are expected
        params = {"status": 'fake'}
        shares = self.shares_client.list_shares_with_detail(params)['shares']
        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('d24a438e-4622-48ac-993e-a30d04746745')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_get_snapshot(self):

        # get snapshot
        get = self.shares_client.get_snapshot(self.snap["id"])['snapshot']

        # verify keys
        expected_keys = ["status", "links", "share_id", "name",
                         "share_proto", "created_at",
                         "description", "id", "share_size"]
        actual_keys = get.keys()
        [self.assertIn(key, actual_keys) for key in expected_keys]

        # verify data
        msg = "Expected name: '%s', actual name: '%s'" % (self.snap_name,
                                                          get["name"])
        self.assertEqual(self.snap_name, get["name"], msg)

        msg = ("Expected description: '%s', "
               "actual description: '%s'" % (self.snap_desc,
                                             get["description"]))
        self.assertEqual(self.snap_desc, get["description"], msg)

        msg = ("Expected share_id: '%s', "
               "actual share_id: '%s'" % (self.shares[0]["id"],
                                          get["share_id"]))
        self.assertEqual(self.shares[0]["id"], get["share_id"], msg)

    @decorators.idempotent_id('9fae88a5-dd95-40ba-96e2-ac3694cf455f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_list_snapshots(self):

        # list share snapshots
        snaps = self.shares_client.list_snapshots()['snapshots']

        # verify keys
        keys = ["id", "name", "links"]
        [self.assertIn(key, sn.keys()) for sn in snaps for key in keys]

        # our share id in list and have no duplicates
        gen = [sid["id"] for sid in snaps if sid["id"] in self.snap["id"]]
        msg = "expected id lists %s times in share list" % (len(gen))
        self.assertEqual(1, len(gen), msg)

    @decorators.idempotent_id('84013334-5985-4067-8b54-4c633f6022f3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_list_snapshots_with_detail(self):

        # list share snapshots
        snaps = self.shares_client.list_snapshots_with_detail()['snapshots']

        # verify keys
        keys = ["status", "links", "share_id", "name",
                "share_proto", "created_at",
                "description", "id", "share_size"]
        [self.assertIn(key, sn.keys()) for sn in snaps for key in keys]

        # our share id in list and have no duplicates
        gen = [sid["id"] for sid in snaps if sid["id"] in self.snap["id"]]
        msg = "expected id lists %s times in share list" % (len(gen))
        self.assertEqual(1, len(gen), msg)



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF

LATEST_MICROVERSION = CONF.share.max_api_microversion


def generate_long_description(des_length=256):
    random_str = ''
    base_str = 'ABCDEFGHIGKLMNOPQRSTUVWXYZabcdefghigklmnopqrstuvwxyz'
    length = len(base_str) - 1
    for i in range(des_length):
        random_str += base_str[random.randint(0, length)]
    return random_str


@ddt.ddt
class ShareTypesNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(ShareTypesNegativeTest, cls).resource_setup()
        cls.st = cls.create_share_type()
        cls.st2 = cls.create_share_type()

    @decorators.idempotent_id('d6a6ac4d-6582-408d-ba55-6f5128eb940e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_share_type_with_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.create_share_type,
                          data_utils.rand_name("used_user_creds"),
                          client=self.shares_client)

    @decorators.idempotent_id('857c664f-e634-4865-ba05-bdcd4336725d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_share_type_with_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.delete_share_type,
                          self.st["id"])

    @decorators.idempotent_id('06203276-f6a3-4a07-a014-8749763395d6')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_add_access_to_share_type_with_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.add_access_to_share_type,
                          self.st['id'],
                          self.shares_client.tenant_id)

    @decorators.idempotent_id('08b2d093-2ad8-46aa-8112-81d50547f36d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_remove_access_from_share_type_with_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.remove_access_from_share_type,
                          self.st['id'],
                          self.shares_client.tenant_id)

    @utils.skip_if_microversion_not_supported("2.50")
    @decorators.idempotent_id('4a22945c-8988-43a1-88c9-eb86e6abcd8e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.named_data(
        ('2_50', '2.50', '', None, None),
        (LATEST_MICROVERSION, LATEST_MICROVERSION, '', None, None),
        ('2_50_bad_public', '2.50', None, None, 'not_bool'),
        (f'{LATEST_MICROVERSION}_bad_public', LATEST_MICROVERSION, None, None,
         'not_bool'),
        ('2_50_description', '2.50', None, generate_long_description(256),
         None),
        (f'{LATEST_MICROVERSION}_description', LATEST_MICROVERSION, None,
         generate_long_description(256), None),
    )
    def test_share_type_update_bad_request(
            self, version, st_name, st_description, st_is_public):
        st_id = self.st['id']
        # Update share type
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_shares_v2_client.update_share_type,
                          st_id, st_name, st_is_public, st_description,
                          version)

    @utils.skip_if_microversion_not_supported("2.50")
    @decorators.idempotent_id('7193465a-ed8e-44d5-9ca9-4e8a3c5958e0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.data('2.50', LATEST_MICROVERSION)
    def test_share_type_update_conflict(self, version):
        name_1 = self.st['name']
        st_id_2 = self.st2['id']
        # Update share type
        self.assertRaises(lib_exc.Conflict,
                          self.admin_shares_v2_client.update_share_type,
                          st_id_2, name_1, None, None, version)



# Copyright 2022 NetApp Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareServerMultipleSubnetTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareServerMultipleSubnetTest, cls).skip_checks()
        if not CONF.share.multitenancy_enabled:
            raise cls.skipException('Multitenancy tests are disabled.')
        if not CONF.share.run_share_server_multiple_subnet_tests and not (
                CONF.share.run_network_allocation_update_tests):
            raise cls.skipException(
                'Share server multiple subnets and network allocation '
                'update tests are disabled.')
        if CONF.share.share_network_id != "":
            raise cls.skipException(
                'These tests are not suitable for pre-existing '
                'share_network.')
        utils.check_skip_if_microversion_not_supported("2.70")

    @classmethod
    def resource_setup(cls):
        super(ShareServerMultipleSubnetTest, cls).resource_setup()
        cls.extra_specs = {
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }
        if CONF.share.run_share_server_multiple_subnet_tests:
            cls.extra_specs['share_server_multiple_subnet_support'] = True
        if CONF.share.run_network_allocation_update_tests:
            cls.extra_specs['network_allocation_update_support'] = True
        share_type = cls.create_share_type(extra_specs=cls.extra_specs)
        cls.share_type_id = share_type['id']

        cls.zones = cls.get_availability_zones_matching_share_type(
            share_type)
        if len(cls.zones) == 0:
            msg = ("These tests need at least one compatible "
                   "availability zone.")
            raise cls.skipException(msg)

        cls.share_network = cls.alt_shares_v2_client.get_share_network(
            cls.alt_shares_v2_client.share_network_id)['share_network']
        cls.default_subnet = utils.share_network_get_default_subnet(
            cls.share_network)

    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        not CONF.share.run_share_server_multiple_subnet_tests,
        "Share network multiple subnets tests are disabled.")
    @decorators.idempotent_id('5600bd52-ecb4-47d3-a4e8-3e6565cb0b80')
    def test_create_share_on_multiple_subnets_same_az(self):
        share_network_id = self.create_share_network(
            cleanup_in_class=False)["id"]
        subnet_data = {
            'neutron_net_id': self.default_subnet.get('neutron_net_id'),
            'neutron_subnet_id': self.default_subnet.get('neutron_subnet_id'),
            'share_network_id': share_network_id,
            'availability_zone': self.zones[0],
        }
        subnet1 = self.create_share_network_subnet(**subnet_data)
        subnet2 = self.create_share_network_subnet(**subnet_data)

        share = self.create_share(
            share_type_id=self.share_type_id,
            share_network_id=share_network_id,
            availability_zone=self.zones[0])
        self.assertIn(share['status'], ('creating', 'available'))

        share = self.admin_shares_v2_client.get_share(share['id'])['share']
        share_server = self.admin_shares_v2_client.show_share_server(
            share['share_server_id']
        )['share_server']
        self.assertIn(subnet1['id'],
                      share_server['share_network_subnet_ids'])
        self.assertIn(subnet2['id'],
                      share_server['share_network_subnet_ids'])

        # Delete share
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
        # Delete the subnets
        self.shares_v2_client.delete_subnet(share_network_id, subnet1['id'])
        self.shares_v2_client.delete_subnet(share_network_id, subnet2['id'])

    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        not CONF.share.run_network_allocation_update_tests,
        "Share server network allocation update are disabled.")
    @decorators.idempotent_id('2a9debd5-47a3-42cc-823b-2b9de435a5e4')
    def test_create_share_with_network_allocation_update(self):
        share_network_id = self.create_share_network(
            cleanup_in_class=False)["id"]
        subnet_data = {
            'neutron_net_id': self.default_subnet.get('neutron_net_id'),
            'neutron_subnet_id': self.default_subnet.get('neutron_subnet_id'),
            'share_network_id': share_network_id,
            'availability_zone': self.zones[0],
        }
        subnet1 = self.create_share_network_subnet(**subnet_data)

        share = self.create_share(
            share_type_id=self.share_type_id,
            share_network_id=share_network_id,
            availability_zone=self.zones[0])
        self.assertIn(share['status'], ('creating', 'available'))
        share = self.admin_shares_v2_client.get_share(share['id'])['share']

        waiters.wait_for_subnet_create_check(
            self.shares_v2_client, share_network_id,
            neutron_net_id=subnet_data['neutron_net_id'],
            neutron_subnet_id=subnet_data['neutron_subnet_id'],
            availability_zone=self.zones[0])
        subnet2 = self.create_share_network_subnet(**subnet_data)

        waiters.wait_for_resource_status(
            self.admin_shares_v2_client, share['share_server_id'],
            constants.SERVER_STATE_ACTIVE,
            resource_name="share_server",
            status_attr="status")
        share_server = self.admin_shares_v2_client.show_share_server(
            share['share_server_id']
        )['share_server']

        self.assertIn(subnet1['id'],
                      share_server['share_network_subnet_ids'])
        self.assertIn(subnet2['id'],
                      share_server['share_network_subnet_ids'])

        # Delete share
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
        # Delete subnets
        self.shares_v2_client.delete_subnet(share_network_id, subnet1['id'])
        self.shares_v2_client.delete_subnet(share_network_id, subnet2['id'])

    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        not CONF.share.run_network_allocation_update_tests,
        "Share server network allocation update are disabled.")
    @decorators.idempotent_id('2624f9a7-660b-4f91-89b8-c026b3bb8d1f')
    def test_share_network_subnet_create_check(self):
        """The share network subnet create check compatibility test."""

        share_network_id = self.create_share_network(
            cleanup_in_class=False)["id"]
        subnet_data = {
            'neutron_net_id': self.default_subnet.get('neutron_net_id'),
            'neutron_subnet_id': self.default_subnet.get('neutron_subnet_id'),
            'share_network_id': share_network_id,
            'availability_zone': self.zones[0],
        }
        subnet1 = self.create_share_network_subnet(**subnet_data)

        share = self.create_share(
            share_type_id=self.share_type_id,
            share_network_id=share_network_id,
            availability_zone=self.zones[0]
        )
        self.assertIn(share['status'], ('creating', 'available'))
        waiters.wait_for_subnet_create_check(
            self.shares_v2_client, share_network_id,
            neutron_net_id=subnet_data['neutron_net_id'],
            neutron_subnet_id=subnet_data['neutron_subnet_id'],
            availability_zone=self.zones[0])

        # Delete share
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
        # Delete subnets
        self.shares_v2_client.delete_subnet(share_network_id, subnet1['id'])



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LOG = log.getLogger(__name__)


class SecurityServicesNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(SecurityServicesNegativeTest, cls).resource_setup()
        # create share_type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

    @decorators.idempotent_id('f5cdf074-f5d4-4d9e-990b-c3d9385dfc2b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_security_service_with_empty_type(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.create_security_service, "")

    @decorators.idempotent_id('6a7efebc-989a-42b3-a971-c28af1e209f5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_security_service_with_wrong_type(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.create_security_service,
                          "wrong_type")

    @decorators.idempotent_id('2935de41-40b1-4907-8ab5-f4921f670bfd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_security_service_without_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.get_security_service, "")

    @decorators.idempotent_id('34e923a4-e5b4-4375-88b4-8bb05e8f010d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_security_service_with_wrong_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.get_security_service,
                          "wrong_id")

    @decorators.idempotent_id('887e0c2c-6658-442d-92e7-3b0c86a2dfcd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_security_service_without_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.delete_security_service, "")

    @decorators.idempotent_id('c94eb229-5caf-4e81-b8a9-d73c0b63a93b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_security_service_with_wrong_type(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.delete_security_service,
                          "wrong_id")

    @decorators.idempotent_id('60e51b95-af7f-463d-8d8e-6bca4ed3bec8')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_nonexistant_security_service(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.update_security_service,
                          "wrong_id", name="name")

    @decorators.idempotent_id('e4554b3b-bc74-4204-8596-d502149bb408')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_security_service_with_empty_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.update_security_service,
                          "", name="name")

    @decorators.idempotent_id('ab62240a-219c-4fa7-b1ca-08f91bec76f0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    def test_try_update_invalid_keys_sh_server_exists(self):
        ss_data = utils.generate_security_service_data()
        ss = self.create_security_service(**ss_data)

        sn = self.shares_client.get_share_network(
            self.shares_client.share_network_id)['share_network']
        fresh_sn = self.create_share_network(
            add_security_services=False,
            neutron_net_id=sn["neutron_net_id"],
            neutron_subnet_id=sn["neutron_subnet_id"])

        self.shares_client.add_sec_service_to_share_network(
            fresh_sn["id"], ss["id"])

        # Security service with fake data is used, so if we use backend driver
        # that fails on wrong data, we expect error here.
        # We require any share that uses our share-network.
        try:
            self.create_share(share_type_id=self.share_type_id,
                              share_network_id=fresh_sn["id"],
                              cleanup_in_class=False)
        except Exception as e:
            # we do wait for either 'error' or 'available' status because
            # it is the only available statuses for proper deletion.
            LOG.warning("Caught exception. It is expected in case backend "
                        "fails having security-service with improper data "
                        "that leads to share-server creation error. "
                        "%s", str(e))

        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.update_security_service,
                          ss["id"],
                          user="new_user")

    @decorators.idempotent_id('288dbf42-ee22-4445-8363-7ebb1c3d89c9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_deleted_security_service(self):
        data = utils.generate_security_service_data()
        ss = self.create_security_service(**data)
        self.assertLessEqual(data.items(), ss.items())

        self.shares_client.delete_security_service(ss["id"])

        # try get deleted security service entity
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.get_security_service,
                          ss["id"])
